#!/usr/local/bin/webwork-perl

## $Id: TALogin.pl,v 1.5 2001/06/18 15:21:51 sam Exp $

####################################################################
# Copyright @ 1995-1998 University of Rochester
# All Rights Reserved
####################################################################


## This file is TALogin.pl
## It provides access to utilities for building, correcting and viewing problem sets
##
## It is called from a form with inputs
## 'user',
##^'key'
## 'course'

use lib '.'; use webworkInit; # WeBWorKInitLine
use CGI qw(:standard);
use Global;
use Auth;
use strict;

my $logTimingData = 0;
my $beginTime;

# begin Timing code
if ($logTimingData == 1) {
	use Benchmark;
	$beginTime = new Benchmark;
}
# end Timing code


my $cgi = new CGI;
my %inputs = $cgi->Vars();

# get information from CGI inputs  (see also below for additional information)

	my $Course 		= $cgi->param('course');
	my $User 		= $cgi->param('user');
	my $Session_key 	= $cgi->param('key');

# verify that information has been received
	unless($Course && $User && $Session_key) {
		&Global::error("Script Error","The script TALogin.pl did not receive the proper input data.","","");
		die "The script TALogin.pl did not receive the proper input data.";
		}

# establish environment for this script

	&Global::getCourseEnvironment($Course);


	my $cgiURL   			= $Global::cgiWebworkURL;
	my $courseScriptsDirectory 	= $Global::courseScriptsDirectory;
	my $databaseDirectory 		= $Global::databaseDirectory;
	my $htmlURL       		= $Global::htmlURL;
	my $scriptDirectory   		= $Global::scriptDirectory;

	require "${scriptDirectory}$Global::DBglue_pl";
	require "${scriptDirectory}$Global::classlist_DBglue_pl";
	require "${scriptDirectory}$Global::FILE_pl";
	require "${scriptDirectory}$Global::HTMLglue_pl";

# log access
	&Global::log_info('', query_string);


	my $permissionsFile = &Global::getCoursePermissionsFile($Course);
	my $permissions = &get_permissions($User, $permissionsFile);
	my $keyFile = &Global::getCourseKeyFile($Course);

#verify session key
	&verify_key($User, $Session_key, "$keyFile", $Course);

# verify permissions are correct
	if (($permissions != $Global::instructor_permissions) and ($permissions != $Global::TA_permissions)){
		print "permissions = $permissions TA_permissions = $Global::TA_permissions\n";
		print &html_NO_PERMISSION;
		exit(0);
		}
# get the rest of the information from the submitted form
	# nothing further to get in this case

	my %availableSets = ();
	my @sortedSetNames = ();
	my @availableStudents = ();

# get information from the data base and create database if necessary

if ( -e "${databaseDirectory}$Global::database" ) {
	# a list of all of the currently created sets:
	# Keys contains the set numbers and the value is a representative psvn
	%availableSets = &getAllProbSetNumbersHash;
	@sortedSetNames = &sortSetNamesByDueDate(\%availableSets);
	@availableStudents = @{getAllLoginNamesSortedByName()};
}

# print HTML text
print &htmlTOP("WeBWorK TA Utilities");

# print navigation buttons

print $cgi->a( { -href=>"${cgiURL}login.pl?user=$User&key=$Session_key&course=$Course" },

			$cgi->img({ #  -name=>'upImg', # name is not a legal attribute for images
					  -src=>"${Global::upImgUrl}",
					  -align=>'right',
					  -border=>'1',
					  -alt=>'[Up]'
					})
			),
	$cgi->p;

print "\n",
	 $cgi->hr, $cgi->br,
	 "\n\n", $cgi->h3({ -align=>'left' }, "WeBWorK Utility programs for $Course"), "\n",
	 $cgi->p,
	 "From this page, you can monitor the progress of students and view their problems.",
	 $cgi->hr,
	 $cgi->center(
		$cgi->a({-href=>"#Student"}, "Student Progress")
	 );

my %sethash; #popup_menu takes a hash for the labels to associate each label with a specific value
my %labels = ();

if ( -e "${databaseDirectory}$Global::database" ) {
### show only build problem set part if data base does not exist

# course data form
print heading('Section', "1. View statistical data on problem sets for the whole course and sections in $Course"),
	$cgi->startform(-action=>"${cgiURL}profCourseData.pl"),
	$cgi->submit(-value=>'View Course and Section Data'), "\n";


foreach (@sortedSetNames) {
	$sethash{$_} = "Set $_"; #insert the word Set before each set name
}

print $cgi->popup_menu(-name=>'setNo', -values=>\@sortedSetNames, -labels=>\%sethash), "\n",
	hiddens('user', 'key', 'course'), "\n",
	$cgi->endform(), "\n",
	"This allows you to see the average score on a set and on individual problems for
	the course as a whole and also by section.  The average number of incorrect
	attempts per problem is also displayed.";

# course recitation data form
print heading('Recitation', "2. View statistical data on problem sets for the whole course and recitations in $Course"),
	$cgi->startform(-action=>"${cgiURL}profCourseRecData.pl"),
	$cgi->submit(-value=>'View Course and Recitation Data'), "\n";



print $cgi->popup_menu(-name=>'setNo', -values=>\@sortedSetNames, -labels=>\%sethash), "\n",
	hiddens('user', 'key', 'course'),
	$cgi->endform(), "\n",
	"This allows you to see the average score on a set and on individual problems for
	the course as a whole and also by recitation.  The average number of incorrect
	attempts per problem is also displayed.";



# classlist.pl form -- view set data for the entire class

print heading('Student', "3. View students progress on problem sets in $Course:"),
	$cgi->startform(-action=>"${cgiURL}classlist.pl"),
	$cgi->submit(-value=>'View Students Progress'), "\n",
	$cgi->popup_menu(-name=>'setNo', -values=>\@sortedSetNames, -labels=>\%sethash), "\n",
	$cgi->br, "\n",
	"Order students \n",
	$cgi->radio_group(-name=>'sortOrder', -values=>['byCourse','bySection','byRecitation'], -labels=>{ 'byCourse'=>'Alphabetically', 'bySection'=>'by section ', 'byRecitation'=>'by recitattion '}),
	hiddens('user', 'key', 'course'),
	$cgi->hidden(-name=>'pCl', -value=>1), "\n",
	$cgi->endform(), "\n",
	"This also allows you to find the psvn for a given student and to view
	the problems as viewed by the student.  You can see the exact answer expected
	by WeBWorK and also, if you wish, answer the problem for the student.";


} ### end of if ( -e "${databaseDirectory}$Global::database" )
### show only build problem set part if data base does not exist


print &htmlBOTTOM("TALogin.pl", \%inputs);

# begin Timing code
if ($logTimingData == 1) {
	my $endTime = new Benchmark;
	&Global::logTimingInfo($beginTime,$endTime,'TALogin.pl',$Course,$User);
}
# end Timing code
exit;

################################################################################
#
# HELPFUL METHODS
#
################################################################################

#returns the horizontal line, blue square, and heading for each option
#in the list, eliminating the need to repeat this code each time it is needed
#this does NOT PRINT the info, so that it can be incorporated into other text
sub heading {
        my $link = shift; #name for internal link (used for shortcuts on some pages)
        my $text = shift; #text used as the main heading

        return "\n",
                 $cgi->p, "\n",
                 $cgi->a({-name=>$link}, ''), "\n",
                 $cgi->hr({ -noshade=>undef }), "\n",
                 $cgi->h4({ -align=>'LEFT' },
                                "\n" . $cgi->img({-src=>"$Global::bluesquareImgUrl", -border=>1, -alt=>'' }) . "\n" .
                                $text ), "\n";
}

#prints hidden form fields for each of given cgi parameters
#if a given parameter does not exist, a note is placed in the html to that affect
#this does NOT PRINT the info, so that it can be incorporated into other text
sub hiddens {
	my @params = @_;
	my $out;

	foreach my $param (@params) {
		if (exists $inputs{$param}) {
			$out .= $cgi->hidden(-name=>"$param", -value=>"$inputs{$param}") . "\n";
		} else {
			$out .= $cgi->p . "\nExpected cgi parameter $param does not exist or is empty";
		}
	}
	$out;
}