#!/usr/local/bin/webwork-perl
############################################################
#	File: alias.pl
#	Copyright:  WeBWorK 1997  all rights reserved
#	Description: 	This script will read, interpret, and output certain files
#                    not normally accessible by the browser.  It can be used
#                    to provide real time creation of .gif files
############################################################


use lib '.'; use webworkInit; # WeBWorKInitLine
use strict;

use CGI qw(:standard);
use Global;
use Auth;

# begin Timing code
use Benchmark;
my $beginTime = new Benchmark;
# end Timing code


&CGI::ReadParse;
my %inputs = %main::in;

# get information from CGI inputs  (see also below for additional information)
	my $Course 			= param('course');
	my $User 			= param('user');
	my $Session_key    	= param('key');
	my $psvn  			= param('probSetKey');  #psvn stands for Problem Set Version Number
	my $probNum			= param('probNum');
	my $mode			= param('Mode');
	my $previousProbNum = $probNum-1 if defined($probNum);
	my $nextProbNum 	= $probNum+1 if defined($probNum);


# verify that information has been received
	unless($Course && $User && $Session_key && $psvn) {
		&Global::error("Script Error","The script alias.pl did 
						not receive the proper input data.","","");
#		die "Content-type: text/html\n\nThe script processProblem.pl did
#						not receive the proper input data.";
	}

# establish environment for this script
	&Global::getCourseEnvironment($Course);

	my $databaseDirectory 		= getCourseDatabaseDirectory(); 		#$Global::databaseDirectory;
	my $htmlDirectory 	  		= getCourseHtmlDirectory();     		#$Global::htmlDirectory;
	my $htmlURL           		= getCourseHtmlURL();           		#$Global::htmlURL;
	my $scriptDirectory   		= getWebworkScriptDirectory();   		#$Global::scriptDirectory;
	my $templateDirectory 		= getCourseTemplateDirectory(); 		#$Global::templateDirectory;
	my $courseScriptsDirectory 	= getCourseScriptsDirectory(); 	#$Global::courseScriptsDirectory;
 	
 	require "${scriptDirectory}$Global::DBglue_pl";
 	
 	
 	
 		my $permissionsFile 		= &Global::getCoursePermissionsFile($Course);
	my $keyFile 					= &Global::getCourseKeyFile($Course);

# log access
	&Global::log_info('', query_string);

#verify session key
	&verify_key($User, $Session_key, "$keyFile", $Course);

# get the rest of the information from the CGI script
	my	$fileName = param('fileName');

# getinformation about the problem set (from the webwork-database)
# and begin constructing the environment for constructing and displaying the problem

	&attachProbSetRecord($psvn);
	
my	$probFolder	=	getProblemFileName($probNum,$psvn);
	$probFolder =~ s|/[^/]+$|/|;  # trim off the file name
	$fileName =~ /\.([^.]+)$/;
my $fileType = $1; 



if ($fileType eq 'gif') {
	my $filePath =  "${templateDirectory}$probFolder$fileName";
	open(INPUT, "<$filePath")||  print "content-type: text/plain\n\nCan't open $filePath\n";                        
	
	print "content-type: image/gif\n\n";
	my $input ="";
	while (read INPUT, $input, 16384) {
		print $input;
	}
	close(INPUT);
} elsif ($fileType eq 'eps') {   # this doesn't quite work yet
	$fileName =~ s/\.eps/\.gif/;   # we will read the .gif file and convert it to postscript
	my $filePath =  "${templateDirectory}$probFolder$fileName";

	open(INPUT, "cat $filePath | /usr/math/bin/giftopnm |/usr/math/bin/pnmtops|")||  print "content-type: text/plain\n\nCan't open $filePath\n";                        
	print  "content-type: application/postscript\n\n";

 	my $input ="";
 	while (read INPUT, $input, 16384) {
 		print  $input;
 	}
 	close(INPUT);
} elsif ($fileType eq 'html') {
    my $filePath =  "${templateDirectory}$probFolder$fileName";
    open(INPUT, "<$filePath")||  print "content-type: text/plain\n\nCan't open $filePath\n";                        
    print "content-type: text/html\n\n";
    my $input ="";

    while (read INPUT, $input, 16384) {
		print $input;
    }
	close(INPUT);
} else {
	&Global::error("Can't handle |$fileType| filetype yet");
}

# begin Timing code
my $endTime = new Benchmark;
&Global::logTimingInfo($beginTime,$endTime,"alias.pl",$Course,$User);
# end Timing code
exit;

### DONE ###
