#!/usr/local/bin/webwork-perl

## $Id: login.pl,v 1.5 2001/06/18 15:21:51 sam Exp $

####################################################################
# Copyright @ 1995-1998 University of Rochester
# All Rights Reserved
####################################################################


use lib '.'; use webworkInit; # WeBWorKInitLine

## login.pl
## password authorization script for the WeBWorK project
##
## To Do:
##	* make sure submitted class value is in list of valid classes?
##	* add -T for taint checks, eventually...


require 5.001;
use strict;

use Global;
use Auth;
use CGI qw(:standard);

# begin Timing code
use Benchmark;
my $beginTime = new Benchmark;
# end Timing code

my $Course	= param('course');
my $User	= param('user');
my $Passwd	= param('passwd');
my $Key		= param('key');
my $scriptURL = param('SCRIPTING_FILE');   # script to execute after fast relogin

# We may not have course info yet, so only get course environment if we
# know what course we're getting info for!

&Global::getCourseEnvironment(param('course')) if param('course');

my $scriptDirectory = $Global::scriptDirectory;
require "${scriptDirectory}$Global::HTMLglue_pl";

my $Passwd_file     = &getCoursePasswordFile($Course);
my $Key_file        = &getCourseKeyFile($Course);
my $permissionsFile = &getCoursePermissionsFile($Course);
my $practiceUser = $Global::practiceUser;

# If a valid key is specified, we've just changed password
# so no need to re-authorize

if ($Key && $User && $Course)  {
    &verify_key($User, $Key, $Key_file, $Course);
#   &record_login;
    &success_form($Key);
    exit;
}

## No course specified?  You lose.
unless($Course ) {
    my $error_msg = remote_host().'   '.user_agent().'   '.query_string();
    &wwerror("$0, missing course data. Refered by:" . referer(),
        "The script did not receive the proper input data. No WeBWorK course was defined",'','',$error_msg
	);
}


## 'course' specified, but we also need 'user' and 'passwd' fields
unless ($User && $Passwd) {
    &Global::error('', "Can't read $Passwd_file") unless (-r $Passwd_file);
    &login_form;
    exit;
}

## If the user is a practice user, check that no one else is currently using
## that practice account

if ($User =~ /^$practiceUser/) {
    my @currentUsers = get_current_users_from_passwordDB($Passwd_file);
    my $user;

    my $is_valid = 0;
    foreach $user (@currentUsers) {    ##check to see if $User is in password file
        if ($user eq $User) {
           $is_valid = 1;
           last;
        }
    }
    unless ($is_valid) {
        &failure_form;
        exit;
    }

    if (-e $Key_file) {  ## if the keys files does not exist, they can be no conflict
        my @activeUsers = get_active_users_from_keysDB($Key_file);
        my $is_active = 0;
        foreach $user (@activeUsers) {     ##check to see if $User is currently active
            if ($user eq $User) {
                $is_active = 1;
                $is_active = 0 if $User eq "practice666"; #code for inspections
                last;
            }
        }
        if ($is_active) {
            &practice_failure_form;
            exit;
        }
    }
}


## we've got all the info we need, now do the authorization
if (&verify_password($User, $Passwd, $Passwd_file)) {
    my $key = &new_key($User, $Key_file);
    if (defined $scriptURL ) {  # we are doing a fast relogin
    	&fast_login_form($key);

	    }
    else {
	    &record_login;   #Mike Gage 8/31/96
	    &success_form($key);
    	}
}
else {
    &failure_form;
}

# begin Timing code
my $endTime = new Benchmark;
&Global::logTimingInfo($beginTime,$endTime,"login.pl",$Course,$User,remote_host(),user_agent());
# end Timing code

exit;

####################

sub login_form {
    my $course = shift;
    my $loginURL = getLoginURL();

    print &htmlTOP('WeBWorK Login Page', '', 'Pragma: no-cache'),
	  hr,
	  h1('WeBWorK Login Page'),
	  "Please enter your username and password for ",
	  b($Course), " below: ",
	  p,
	  start_form('POST', "$loginURL"),
	  hidden('course'),
	  textfield('user', '', 18), " Username", br,
	  password_field('passwd', '', 18), " Password",
	  i(" (will not be echoed)"), br,
	  p,
	  submit('Continue'),
	  end_form, end_html;
}


sub success_form {
    my $key = shift;
    my $user_title = "user ";

    param('key', $key);  # avoid CGI.pm's sticky hidden field values

    my $permissions = &get_permissions($User,$permissionsFile);
    if ($permissions == $Global::instructor_permissions) {
	$user_title  = "instructor ";
    }
	if ($permissions == $Global::TA_permissions) {
	$user_title  = "TA ";
    }
    print &htmlTOP('Login Valid');

    # create outer table: left column is status message + buttons,
    # right-column contains info (message of the day, tips, etc.)
    print qq{<table cellspacing="10"><tr>};

    # begin left column
    print qq{<td valign="top" width="70%">},
        h1('Login Valid'),
	"Welcome, $user_title ", b($User), ". ",
        "You may begin working on problem sets for ",
	b($Course), " or do other tasks.\n";

    if ($permissions == $Global::instructor_permissions) {
	print start_form('POST', "${Global::cgiWebworkURL}profLogin.pl"),
	    p,"\n",
	    hidden('course'), "\n",
	    hidden('user'),   "\n",
	    hidden('key'),    "\n",
	    submit("Enter Professor's Page"),
	    end_form;
    }

    if ($permissions == $Global::TA_permissions) {
	print start_form('POST', "${Global::cgiWebworkURL}TALogin.pl"),
	    p,"\n",
	    hidden('course'), "\n",
	    hidden('user'),   "\n",
	    hidden('key'),    "\n",
	    submit("Enter TA's Page"),
	    end_form;
    }

    print start_form('POST', "$Global::welcome_CGI"),
	p,"\n",
	hidden('course'), "\n",
	hidden('user'),   "\n",
	hidden('key'),    "\n",
	submit('Begin Problem Sets'),
	end_form,
	p;

    print start_form('POST', "${Global::cgiWebworkURL}change-password.pl"),
	hidden('course'), "\n",
	hidden('user'),   "\n",
	hidden('key'),    "\n",
	submit('Change Password'),
	end_form;

    if ($Global::allowStudentToChangeEMAddress) {
        print start_form('POST', "${Global::cgiWebworkURL}change-email.pl"),
	    hidden('course'), "\n",
	    hidden('user'),   "\n",
	    hidden('key'),    "\n",
	    hidden('firstTime','firstTime'),    "\n",
	    submit('Change Email Address'),
	    end_form;
    }
    print qq{</td><td rowspan="2" bgcolor="black"></td>};


    # get "system message of the day"
    my $system_motd = '';
    my $system_motd_file = &Global::getSystemMOTDFile();
    if (-r $system_motd_file) {
	open(MOTD, $system_motd_file);
	while (<MOTD>) {
	    $system_motd .= $_;
	}
	close(MOTD);
    }
    $system_motd = $system_motd || 'None.';
    
    # get "course message of the day"
    my $course_motd = '';
    my $course_motd_file = &Global::getCourseMOTDFile($Course);
    if (-r $course_motd_file) {
	open(MOTD, $course_motd_file);
	while (<MOTD>) {
	    $course_motd .= $_;
	}
	close(MOTD);
    }
    $course_motd = $course_motd || 'None.';
    
    # now create separate "info" table in right-column
    print qq{<td valign="top"><table border cellspacing="5" cellpadding="5">},

	# motd row
	qq{<tr><td valign="top"><font face="helvetica,arial">},
	h4('WeBWorK System Message of the day:'),
	$system_motd,
	qq{</font></td></tr><tr>},

	# motd row
	qq{<tr><td valign="top"><font face="helvetica,arial">},
	h4("$Course Message of the day:"),
	$course_motd,
	qq{</font></td></tr><tr>},	

	# tips row
	qq{<td valign="top"><font face="helvetica,arial">},
	h4('Tip for this login:'),
	&Global::tip,
#	start_form('POST', "${Global::cgiWebworkURL}tips.pl"),
#	hidden('course'), "\n",
#	hidden('user'),   "\n",
#	hidden('key'),    "\n",
#	'<SMALL>',submit('See all tips'),'</SMALL>',
#       end_form,
	qq{</font></td></tr></table>},  # close inner "tips" table
	qq{</td></tr></table>},  # close outer table
        end_html;
}


sub failure_form {
    print &htmlTOP('Login Invalid', $Global::background_warn_url),
	h1('Login Invalid'),
	"Sorry ", b($User), ", you entered an invalid user/password pair.",
	start_form('POST', url),
	hidden('course'),
	submit('Try again'),
	end_form, end_html;
}

sub practice_failure_form {
    ## first get some information on practice users
    my @activeUsers = get_active_users_from_keysDB($Key_file);
    my @currentUsers = get_current_users_from_passwordDB($Passwd_file);
    my @activePracticeUsers = grep /^$practiceUser/, @activeUsers;
    my @currentPracticeUsers = grep /^$practiceUser/, @currentUsers;
    ## get a list of all non active practice users.
    my (%mark,$user);
    grep($mark{$_}++,@activePracticeUsers);
    my @currentAvailablePracticeUsers = grep(!$mark{$_},@currentPracticeUsers);
    my $numberOfAvailablePracticeUsers = @currentAvailablePracticeUsers;
    my $numberOfCurrentPracticeUsers = @currentPracticeUsers;
    ## get a short list of non active practice users
    @currentAvailablePracticeUsers = @currentAvailablePracticeUsers[0..4] if
      $numberOfAvailablePracticeUsers > 5;

    ## now print form
    print &htmlTOP('Login Failed', $Global::background_warn_url),
	  h1('Login Failed'),
	  'Sorry, the ', b($User),' account is currently being used by another user. ',
	  'You can try another ',  b($practiceUser), ' account or you can try to login to the ',
	  b($User),' account at a later time. <p>',
	  "Currently, $numberOfAvailablePracticeUsers out of a total of $numberOfCurrentPracticeUsers ",
	  b($practiceUser), ' accounts are not in use and are available.  For example<p>';
	foreach $user (@currentAvailablePracticeUsers) {
	    print "$user<br>";
	}
	print '<p>are available',
	  start_form('POST', url),
	  hidden('course'),
	  submit('Try again'),
	  end_form, end_html;
}



sub fast_login_form {
    my $key = shift;
    my $user_title = "user ";

    param('key', $key);  # avoid CGI.pm's sticky hidden field values

    my $permissions = &get_permissions($User,$permissionsFile);
    if ($permissions == $Global::instructor_permissions) {
	$user_title  = "instructor ";
    }
    if ($permissions == $Global::TA_permissions) {
	$user_title  = "TA ";
    }

    print &htmlTOP('Login Valid', $Global::background_okay_url),
        h1('Login Valid'),
	"Fast login for $user_title ", b($User), ". ",
     	start_form('POST', $scriptURL),
     	hidden('key'),    "\n";

     &CGI::ReadParse;  #get the input parameters which need to be passed.
     my $tmp_key;
     foreach $tmp_key (keys %main::in) {           # pass all of the variables from the original calling script except those below:
			next if $tmp_key eq 'key';             # we have replaced the old key
			next if $tmp_key eq 'passwd';          # we DO NOT want to pass this.
			next if $tmp_key eq 'GO';              # the submit button, we don't need this
			next if $tmp_key eq 'SCRIPTING_FILE';  # contained the URL of the original target script now in $scriptURL
			print hidden( $tmp_key , $main::in{$tmp_key} ),"\n";
     }

     print "Press Tab and Return -- or click GO button<BR>",
     textfield('dummy','',0,0), submit('GO'), end_form, end_html;
}


sub record_login {

    &Global::getCourseEnvironment($Course);
	my $logsDirectory = getCourseLogsDirectory();
    my $logfile = "${logsDirectory}login.log";
    my $timestamp = scalar localtime;

    $timestamp =~ /^\w+\s(.*)\s/;
    $timestamp = $1;

    open(LOG, ">>$logfile") || warn "$0: can't write to $logfile!\n";
    print LOG "$timestamp $User on ", remote_host(),
	       " (", user_agent(), ")\n";
    close(LOG);
    chmod 0660, "$logfile";
}
