#!/usr/local/bin/webwork-perl

## This file is profAddStudent.pl
## It provides access to utilities for building problem sets
##

####################################################################
# Copyright @ 1995-1999 University of Rochester
# All Rights Reserved
####################################################################

use lib '.'; use webworkInit; # WeBWorKInitLine

use CGI qw(:standard);
use Global;
use Auth;
use strict;
use TimeLocal;

my $logTimingData = 0;
my $beginTime;

# begin Timing code
if ($logTimingData == 1) {
	use Benchmark;
	$beginTime = new Benchmark;
}
# end Timing code



my $cgi = new CGI;
my %inputs = $cgi->Vars();

# get information from CGI inputs  (see also below for additional information)

my $Course 			= $inputs{'course'};
my $User 			= $inputs{'user'};
my $Session_key 	= $inputs{'key'};

# verify that information has been received
unless($Course && $User && $Session_key) {
	&wwerror("$0","The script did not receive the proper input data.","","");
}

# establish environment for this script

&Global::getCourseEnvironment($inputs{'course'});


my $cgiURL   				= getWebworkCgiURL;
my $courseScriptsDirectory 	= getCourseScriptsDirectory;
my $databaseDirectory 		= getCourseDatabaseDirectory;
my $htmlURL       			= getCourseHtmlURL;
my $scriptDirectory   		= getWebworkScriptDirectory;
my $templateDirectory		= getCourseTemplateDirectory;
my $dat                     = getDat;
my $dd                      = getDirDelim;

require "${scriptDirectory}$Global::DBglue_pl";
require "${scriptDirectory}$Global::classlist_DBglue_pl";
require "${scriptDirectory}$Global::FILE_pl";
require "${scriptDirectory}HTMLglue.pl";
require "${scriptDirectory}$Global::buildProbSetTools";

# log access
&Global::log_info('', query_string);

my $passwordFile = &Global::getCoursePasswordFile($inputs{'course'});
my $permissionsFile = &Global::getCoursePermissionsFile($inputs{'course'});
my $permissions = &get_permissions($inputs{'user'}, $permissionsFile);
my $keyFile = &Global::getCourseKeyFile($inputs{'course'});

#verify session key
	&verify_key($inputs{'user'}, $inputs{'key'}, $keyFile, $inputs{'course'});

# verify permissions are correct
	if ($permissions != $Global::instructor_permissions ) {
		print "permissions = $permissions instructor_permissions = $Global::instructor_permissions\n";
		print &html_NO_PERMISSION;
		exit(0);
		}
# get the rest of the information from the submitted form


my $number 			= $inputs{'number'};
my $page 			= $inputs{'page'};
my @setNumbers =();
@setNumbers = $cgi -> param('setNo' );

my $setDefinitionFile;
my $setNumber;
my ($time1, $time2, $time3);

my @added_student_logins =();

$number = min(100, int($number));
$number = max(1, $number);

my $action = $inputs{Action};

if ($action eq 'return') {
	print_add_student_form();
}
elsif ($action eq 'Add Students') {
	$time1 = &unformatDateAndTime($inputs{'OpenDate'});
	$time2 = &unformatDateAndTime($inputs{'DueDate'});
	$time3 = &unformatDateAndTime($inputs{'AnswerDate'});

	test_inputs();
	my $msg1 = add_students();
	my $msg2 = build_sets();
	change_dates();
	#trim_msg(\$msg2);
	print_html($msg1,$msg2);
}


# begin Timing code
if ($logTimingData == 1) {
	my $endTime = new Benchmark;
	&Global::logTimingInfo($beginTime,$endTime,"dataMunger.pl",$Course,$User);
}
# end Timing code
exit;

## end of main script

sub print_add_student_form {

	# print HTML text
	print &htmlTOP("Add Student(s) Page");

	# print navigation buttons
	print qq!
	<A HREF="${cgiURL}profLogin.pl?user=$inputs{'user'}&amp;key=$inputs{'key'}&amp;course=$inputs{'course'}">
	<IMG SRC="${Global::upImgUrl}" align="right" BORDER=1 ALT="[Up]"></A><p>
	!;

	print <<EOF;
<HR><BR>

<h3 align="left">Add Student(s) to $Course</h3>

From this page you add from one to five students to $Course.
EOF

	print qq! <FORM METHOD = "POST" ACTION= "${cgiURL}profAddStudent.pl"> \n!;

	#start_form('POST', "${Global::cgiWebworkURL}profAddStudent.pl"),
	print
	'If you want to add more at one time, enter the total number ',
	textfield('number',5,3), "\n",
	'and hit ';
	print <<EOF;
	<INPUT TYPE=SUBMIT NAME='Action' VALUE='return'>
EOF



	print <<EOF;
<p>Usually this page is used
to add students who have entered the course after it has started.  This will add students
to the claslist database and initialize passwords for them.  Also select the set(s) to build for
the student(s). All sets which have already been built for students in the course are listed.
Finally enter open, due, and answer dates (or use the defaults).
These dates will be used for all students (listed on this page) and all sets.
<p>
The Student ID is often a Social Security Number. Note that both the student ID and the Login Name
must be different than any other student's Id or Login Name. (WeBWorK will check for duplicates.)
<p>
EOF






	#print qq! <FORM METHOD = "POST" ACTION= "${cgiURL}profAddStudent.pl"> \n!;

		print qq! <TABLE BORDER="1" CELLPADDING="0" CELLSPACING="1" > !;

		print "<TR>\n";
			print &formatHeaderCell("Last Name");
			print &formatHeaderCell("First Name");
			print &formatHeaderCell("Student ID" );
			print &formatHeaderCell("Login Name");
			print &formatHeaderCell("Section");
			print &formatHeaderCell("Recitation");
			print &formatHeaderCell("Email Address" );
			print &formatHeaderCell("Comment" );


		print "</TR>\r\n";
		my ($i,$sln,$sfn,$sid,$sli,$sec,$rec,$sea,$com);

		for ($i=1; $i <= $number; $i++) {
		    print "<TR>\r\n";
			$sln ='';
			$sln = stripWhiteSpace($inputs{"StudentLastName$i"}) if defined $inputs{"StudentLastName$i"};
			print &formatDataCell("StudentLastName$i","$sln", '16');
			$sfn ='';
			$sfn = stripWhiteSpace($inputs{"StudentFirstName$i"}) if defined $inputs{"StudentFirstName$i"};
			print &formatDataCell("StudentFirstName$i", "$sfn", '13');
			$sid ='';
			$sid = stripWhiteSpace($inputs{"StudentID$i"}) if defined $inputs{"StudentID$i"};
			print &formatDataCell("StudentID$i", "$sid", '12');
			$sli ='';
			$sli = stripWhiteSpace($inputs{"LoginName$i"}) if defined $inputs{"LoginName$i"};
			print &formatDataCell("LoginName$i","$sli", '8');
			$sec ='';
			$sec = stripWhiteSpace($inputs{"Section$i"}) if defined $inputs{"Section$i"};
			print &formatDataCell("Section$i", "$sec", '10');
			$rec ='';
			$rec = stripWhiteSpace($inputs{"Recitation$i"}) if defined $inputs{"Recitation$i"};
			print &formatDataCell("Recitation$i", "$rec", '10');
			$sea ='';
			$sea = stripWhiteSpace($inputs{"EmailAddress$i"}) if defined $inputs{"EmailAddress$i"};
			print &formatDataCell("EmailAddress$i", "$sea", '24');
			$com ='';
			$com = stripWhiteSpace($inputs{"Comment$i"}) if defined $inputs{"Comment$i"};
			print &formatDataCell("Comment$i", "$com", '16');


			print "</TR>\r\n";
		}
		print qq! </TABLE >!;

		print qq! <p>Select the set(s) and enter appropriate dates (or use the defaults)
		if you want to build sets for the above student(s). You can submit this form multiple
		times to build sets with different due dates.  If you do not select any sets
		(or select none) no sets will be built. If you want the new students to
		have the same due dates other students have, do not build sets here but after
		adding the students, go to the Build Problem Set page and build sets there.<p>!;
		my %availableSets = &getAllProbSetNumbersHash;
		my @sortedSetNames = &sortSetNamesByDueDate(\%availableSets);
		my $ind;

		# enter the available set numbers



	## enter open, due and answer dates
		# format the dates properly
		my $timeOpen = time;
		$timeOpen = int($timeOpen/3600)*3600;           ## previous hour
		my $timeDue = $timeOpen + 14*24*3600 +3600;     ## 14 days from next hour
		my $timeAnswer = $timeOpen + 15*24*3600+ 3600;  ## 15 days from next hour

		my $OpenDate       = &formatDateAndTime($timeOpen);
		my $DueDate        = &formatDateAndTime($timeDue);
		my $AnswerDate     = &formatDateAndTime($timeAnswer);

#		print qq! <TABLE BORDER='1' CELLPADDING='0' CELLSPACING='1' > !;
		print qq! <TABLE > !;
			print "<TR>\n";
			print &formatHeaderCell('Sets');


			print &formatHeaderCell('&nbsp;&nbsp;');
			print &formatHeaderCell('Open Date');
			print &formatDataCell("OpenDate",$OpenDate, '17');

			print &formatHeaderCell('&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;');
			print "<TD colspan = 2>\n";
			print 'The drop down lists below are for information only. Selecting items from them does nothing. Enter the section and
recitation data, if any, above.';

			print "<TR>\n";

			print "<TD rowspan = '3'>\n";
			print qq! <SELECT Name = 'setNo' Size =4 multiple>\n!;
				print "<OPTION VALUE = none>none\n";
			for $ind (@sortedSetNames) {
				print "<OPTION VALUE = \"$ind\">Set $ind\n";
			}
			print qq!</SELECT>\n!;
			print "</TD>\n";

			print &formatHeaderCell('&nbsp;&nbsp;');
			print &formatHeaderCell('Due Date');
			print &formatDataCell("DueDate", $DueDate, '17');


			print &formatHeaderCell('&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;');
			print '<td>';
	# Current Sections list
	print $cgi -> popup_menu( -name => 'currentSections',
		-values => ['List of current sections', keys (%{getAllSections()})]
				);

			print '<td>';
	# Current Recitations list
	print $cgi -> popup_menu( -name => 'currentRecitations',
		-values => ['List of current recitations', keys (%{getAllRecitations()})]
				);


			print "<TR>\n";
			print &formatHeaderCell('&nbsp;&nbsp;');
			print &formatHeaderCell('Answer Date' );
			print &formatDataCell("AnswerDate", $AnswerDate, '17');


			print qq! </TABLE >!;


	print qq! <BR><INPUT TYPE=SUBMIT NAME='Action' VALUE='Add Students'>!;

	## resume printing the rest of the form
	print <<EOF;
	<INPUT TYPE='HIDDEN' NAME='page' VALUE='second'>
	<INPUT TYPE='HIDDEN' NAME='user' VALUE="$inputs{'user'}">
	<INPUT TYPE='HIDDEN' NAME='key' VALUE="$inputs{'key'}">
	<INPUT TYPE='HIDDEN' NAME='course' VALUE="$inputs{'course'}">
	</FORM>
EOF

	#print &sessionKeyInputs(\%inputs);  # this duplicates the user, key and course hidden variables and kills some browsers. !!!! 1/21/01/ MEG

	print &htmlBOTTOM("profAddStudent.pl", \%inputs);
}




sub formatDataCell {
	my ($name,$value,$size) = @_;
		# if the data hasn't been entered it appears as a blank:
	$value = '' unless defined($value);

	my $out = qq!
	<TD ALIGN=CENTER VALIGN=MIDDLE >
	<INPUT TYPE="TEXT" NAME="$name" VALUE="$value" SIZE="$size">
	</TD>
	!;
	$out;
	}

sub formatHeaderCell {
	my ($item,$options) = @_;
	$options = "" unless defined($options);
	my $out = qq!
	<TH ALIGN=CENTER VALIGN=MIDDLE $options>
	$item
	</TH>
	!;
	$out;
}


sub add_students {
	my ($i,$sln,$sfn,$sid,$sli,$sec,$rec,$sea,$com);
	my $msg = '';
	my $tmp1_msg = '';
	my $tmp2_msg = '';
	my $tmp3_msg = '';
	my $tmp4_msg = '';
	my $tmp5_msg = '';
	my $tmp6_msg = '';
	my $tmp7_msg = '';
	my $tmp8_msg = '';
	my $uniqueSection;
	my $uniqueRecitation;

	for ($i=1; $i <= $number; $i++) {
		$tmp1_msg = '';
		$tmp2_msg = '';
		$tmp3_msg = '';
		$tmp4_msg = '';
		$tmp5_msg = '';
		$tmp6_msg = '';
		$tmp7_msg = '';
		$tmp8_msg = '';
		$sln ='';
		$sln = stripWhiteSpace($inputs{"StudentLastName$i"}) if defined $inputs{"StudentLastName$i"};
		$sfn ='';
		$sfn = stripWhiteSpace($inputs{"StudentFirstName$i"}) if defined $inputs{"StudentFirstName$i"};
		$sid ='';
		$sid = stripWhiteSpace($inputs{"StudentID$i"}) if defined $inputs{"StudentID$i"};
		$sli ='';
		$sli = stripWhiteSpace($inputs{"LoginName$i"}) if defined $inputs{"LoginName$i"};
		$sec ='';
		$sec = stripWhiteSpace($inputs{"Section$i"}) if defined $inputs{"Section$i"};
		$rec ='';
		$rec = stripWhiteSpace($inputs{"Recitation$i"}) if defined $inputs{"Recitation$i"};
		$sea ='';
		$sea = stripWhiteSpace($inputs{"EmailAddress$i"}) if defined $inputs{"EmailAddress$i"};
		$com ='';
		$com = stripWhiteSpace($inputs{"Comment$i"}) if defined $inputs{"Comment$i"};


		next unless ( $sln or $sfn or $sid or $sli or $sec or $rec or $sea or $com);

		$tmp1_msg = test_entry($sln);
		$tmp2_msg = test_entry($sfn);
		$tmp3_msg = testNewStudentID($sid,$sli);
		$tmp4_msg = testNewStudentLogin($sli,$sid);
		$tmp5_msg = test_entry($sec);
		$tmp6_msg = test_entry($rec);
		$tmp7_msg = test_entry($sea);
		$tmp8_msg = test_entry($com);

		unless (($tmp1_msg eq 'OK') and
				($tmp2_msg eq 'OK') and
				($tmp3_msg eq 'OK') and
				($tmp4_msg eq 'OK') and
				($tmp5_msg eq 'OK') and
				($tmp6_msg eq 'OK') and
				($tmp7_msg eq 'OK') and
				($tmp8_msg eq 'OK')
				) {
			$msg .= "\n    Skipping the entry: $sln, $sfn, $sid, $sli, $sec, $rec, $sea, $com\n";
			$msg .= $tmp1_msg unless $tmp1_msg eq 'OK';
			$msg .= $tmp2_msg unless $tmp2_msg eq 'OK';
			$msg .= $tmp3_msg unless $tmp3_msg eq 'OK';
			$msg .= $tmp4_msg unless $tmp4_msg eq 'OK';
			$msg .= $tmp5_msg unless $tmp5_msg eq 'OK';
			$msg .= $tmp6_msg unless $tmp6_msg eq 'OK';
			$msg .= $tmp7_msg unless $tmp7_msg eq 'OK';
			$msg .= $tmp8_msg unless $tmp8_msg eq 'OK';
			next;
		}
		# add to classlist database

		&CL_putStudentID           ($sid, $sli);
		&CL_putStudentLastName     ($sln, $sli);
		&CL_putStudentFirstName    ($sfn, $sli);
		&CL_putStudentStatus       ('C', $sli);
		&CL_putComment             ($com, $sli);
		&CL_putClassSection        ($sec,$sli);
		&CL_putClassRecitation     ($rec,$sli);
		&CL_putStudentEmailAddress ($sea, $sli);
			# now we can save the classlist record
		&saveCLRecord($sli);

			# these will be set to 1 if the new section or recitation is unique
		$uniqueSection = getAllSections()->{$sec};
		$uniqueRecitation = getAllRecitations()->{$rec};


		$msg .= "\n  At this point,<FONT COLOR='#ff00aa'><B> $sfn $sln is the only person in the section $sec</B></font>" unless $uniqueSection > 1;

		$msg .= "\n  At this point,<FONT COLOR='#ff00aa'><B> $sfn $sln is the only person in the recitation $rec</B></font>" unless $uniqueRecitation >1;

		## add to password and permissions databases
		&new_password($sli, $sid, $passwordFile);
		&put_permissions(0,$sli,$permissionsFile);

		push (@added_student_logins, $sli);

		$msg .= "\nadded $sfn $sln ($sli, $sid) to password database";

	}
	$msg;
}


sub build_sets {

	my $msg = '';
	my $method ='createNewPSVNs';
	my $setDefinitionFile;
	my $outputFormat = 'new_students';

	foreach $setNumber (@setNumbers) {
		next if $setNumber eq 'none';
		$setDefinitionFile = "set${setNumber}.def";
		my $message = &buildProbSetDB($Course,$setDefinitionFile,$method,'',$outputFormat);
		$msg .= "<pre> $message </pre><HR>";
		$outputFormat = 'no_students';  ## if format is all_students, only list them the first time.
	}
	## clean up message --- remove dates which
	$msg =~ s/openDate.*?Loading/Loading/gs;
	$msg;
}

sub change_dates {
	my $sli;
	## return unless we have built some sets
	my $num_of_sets = @setNumbers;

	return unless (($num_of_sets > 1) or (($num_of_sets == 1) and ($setNumbers[0] ne 'none')));
	foreach $sli (@added_student_logins){
		my %setNumberHash=&getAllSetNumbersForStudentLoginHash($sli);
		foreach $setNumber (@setNumbers) {
			next if $setNumber eq 'none';
			next unless defined $setNumberHash{$setNumber};
			my $psvn = $setNumberHash{$setNumber};
			attachProbSetRecord($psvn);
			putOpenDate		($time1, $psvn);
			putDueDate		($time2, $psvn);
			putAnswerDate	($time3, $psvn);
			detachProbSetRecord($psvn);
		}
	}
}
sub test_inputs {
	## verify dates are OK

	if ($time2 < $time1 or $time3 < $time2) {
		&wwerror("ERROR IN DATES.  NO STUDENTS HAVE BEEN ADDED TO $Course and no sets have been built.<P>", "Dates are not in chronological order. The open date: $inputs{'OpenDate'},
		due date: $inputs{'DueDate'}, and answer date: $inputs{'AnswerDate'} must be in chronological order. <P> Use the back button to go back and fix the problem.");
	}

	## verify set definition files are OK

	foreach $setNumber (@setNumbers) {
		next if $setNumber eq 'none';
		$setDefinitionFile = "set${setNumber}.def";
		wwerror ("ERROR IN READING SET DEFINITION FILES.  NO STUDENTS HAVE BEEN ADDED TO $Course and no sets have been built.<P>",
		"Can not read set definition file ${templateDirectory}$setDefinitionFile.
		<P> Fix the problem and then go back and rerun this job.")
		unless -R "${templateDirectory}$setDefinitionFile";
	}
}

sub print_html {
	my ($msg1,$msg2) = @_;

	# print HTML text
	print &htmlTOP("Add Students and Build Problem Sets");

# print navigation buttons
print qq!
	<A HREF="${cgiURL}profAddStudent.pl?user=$inputs{'user'}&key=$inputs{'key'}&amp;course=$inputs{'course'}&amp;number=$inputs{'number'}&amp;Action=return">
	<IMG SRC="${Global::upImgUrl}" align="right" BORDER=1 ALT="[Up]"></A><p>
	!;
	print "<H2><B>INFORMATION ON ADDING STUDENTS TO $Course</B></H2>";
	print "<pre>$msg1</pre>";
	print "<H2><B>INFORMATION ON BUILDING PROBLEM SETS FOR $Course</B></H2>";
	print "<pre>$msg2</pre>";
	print "<H2><B>DONE ADDING STUDENTS AND BUILDING SETS</B></H2>";
	print &htmlBOTTOM("profAddStudent.pl", \%inputs);
}