#!/usr/local/bin/webwork-perl

## This file is profCourseData.pl
## It displays overall data on how students in various sections
## are doing on problems. E.g. the instructor can spot difficult
## problems.
##

####################################################################
# Copyright @ 1995-1998 University of Rochester
# All Rights Reserved
####################################################################

use lib '.'; use webworkInit; # WeBWorKInitLine
use strict;
use Global;
use Auth;
use CGI qw(:standard);

# Timing code
use Benchmark;
my $beginTime = new Benchmark;
# end Timing code


my %inputs;
my ($classID, $setNumber,$User,$Session_key,$batchMode);


	&CGI::ReadParse;
	%inputs = %main::in;

	# get information from CGI inputs  (see also below for additional information)
	$classID     = $inputs{'course'};
	$setNumber = $inputs{'setNo'};
	$User    = $inputs{'user'};
	$Session_key = $inputs{'key'};

	unless ($classID && $User && $Session_key) {
		&wwerror("$0","The script profCourseData.pl did not receive the proper input data.","","", query_string());
	}

# CAUTION:  FOR THIS SCRIPT TO WORK CORRECTLTY,  ALL STUDENTS
#           MUST HAVE BEEN ASSIGNED THE SAME NUMBER OF PROBLREMS

# print "set is $setNumber";
# print "classID is $classID";

&getCourseEnvironment($classID);

my $scriptDirectory   = getWebworkScriptDirectory;
my $databaseDirectory = getCourseDatabaseDirectory;
my $templateDirectory = getCourseTemplateDirectory;
my $cgiURL            = getWebworkCgiURL;

require "${scriptDirectory}$Global::DBglue_pl";
require "${scriptDirectory}$Global::classlist_DBglue_pl";
require "${scriptDirectory}$Global::FILE_pl";
require "${scriptDirectory}$Global::SCRtools_pl";
require "${scriptDirectory}$Global::HTMLglue_pl";

	my $keyFile = &Global::getCourseKeyFile($classID);
	&verify_key($User, $Session_key, $keyFile, $classID);
	my $permissionsFile = &Global::getCoursePermissionsFile($classID);
	my $permissions = &get_permissions($User, $permissionsFile);

   if (($permissions != $Global::instructor_permissions) and ($permissions != $Global::TA_permissions) ) {
		print "permissions = $permissions instructor_permissions= $Global::instructor_permissions\n";
		print &html_NO_PERMISSION;
		exit(0);
	}
my  $DELIM = $Global::delim;

## get rest of data from cgi form
$batchMode = $inputs{'batchMode'};  ## if 1 it means user has choosen to ignore warning message

# get list of problems and values from the database.  Warn user if different students
#have different numbers of problems or different valuse for the same problem
my $cgiMode = 1;
my ($warningMsg,$dueDate, $problemListref,$problemValueListref)
	= &readProblemsAndValuesFromDB($setNumber,0,$cgiMode,$batchMode);

if ($warningMsg ne '') { &warningMsgPage(\%inputs,$warningMsg);}

my @problemList = @$problemListref;
my @problemValueList = @$problemValueListref;
my $noOfProbs = @problemList;
unshift @problemValueList, 0; ##off by 1 problem

my (%NHash,%YHash,%attemptedProbHash);
my (%totalScoreHash,%attemptedSetHash,%allStudentsHash);
my $total_possible;
my @sectionNameArray = keys (%{getAllSections()}) ;
my @excludedSectionNameArray = ();

@excludedSectionNameArray = @Global::excluse_these_sections_from_overall_statistics
  if defined @Global::excluse_these_sections_from_overall_statistics;

&processSectionNameArrays();  ##separate included and excluded sections

## initialize hashes and load up data

my $all = &uniqueCourseName();
&initializeAndLoadUp(\@sectionNameArray,$all);

my $up_page = 'profLogin.pl';
$up_page   = 'TALogin.pl' if $permissions == $Global::TA_permissions; 

# begin printing HTML text
print &htmlTOP("WeBWorK Course Data");

# print navigation buttons
print qq!
<A HREF="${cgiURL}${up_page}?user=$inputs{'user'}&key=$inputs{'key'}&course=$inputs{'course'}">
<IMG SRC="${Global::upImgUrl}" align="right" BORDER=1 ALT="[Up]"></A><p>
!;

print <<EOF;
<HR><BR>

<h3 align="left">Current Statistics for set $setNumber in $classID</h3>

On this page, you can monitor the average progress of students for all sections or
by individual sections. You can identify difficult problems by the percentage of students
who have answered the problem correctly and/or by the average number of incorrect attempts
per problem. All percentages and averages (except \% attempting) are based on the number of
students who have attempted a set or problem, not on the total number of students in the section
or course.  Note that 50\% correct on a partial credit question could mean all students got half
credit or half the students got full credit and half got no credit.<p>
The first two tables do not contain data from "excluded" sections. Excluded sections
typically contain practices uses, T.A.'s, prof's etc.  Data for excluded sections is displayed
at the bottom of the page.
<p>
EOF



## create  $dat arrays with the info. on average scores, etc

my ($sectionName,$i, $dataRow,$item);
my @dataArray =();

## first row

$dataRow = "sections$DELIM \# of students$DELIM \% attempting set $setNumber$DELIM average score$DELIM possible score";
push @dataArray, $dataRow;

## display whole course data

$dataRow = "$all $DELIM $allStudentsHash{$all} $DELIM";  ## all sections and total # of students

$item =0;  ## % of students who have attempted at least one problem in set
$item = int(100*$attemptedSetHash{$all}/$allStudentsHash{$all}+.5) if $allStudentsHash{$all};
$dataRow .= "$item $DELIM";

$item =0;  ## avg score of students who have attempted at least one problem in set
$item = (int($totalScoreHash{$all} *10/$attemptedSetHash{$all}+.5))/10 if $attemptedSetHash{$all};
$item = sprintf '%.1f', $item;
$dataRow .= "$item $DELIM";
$dataRow .= "$total_possible";

push @dataArray, $dataRow;


## display data for sections
foreach $sectionName (@sectionNameArray) {
	$dataRow = "$sectionName $DELIM $allStudentsHash{$sectionName} $DELIM";  ## all sections and total # of students

	$item =0;  ## % of students who have attempted at least one problem in set
	$item = int(100*$attemptedSetHash{$sectionName}/$allStudentsHash{$sectionName}+.5) if $allStudentsHash{$sectionName};
	$dataRow .= "$item $DELIM";

	$item =0;  ## avg score of students who have attempted at least one problem in set
	$item = (int($totalScoreHash{$sectionName} *10/$attemptedSetHash{$sectionName}+.5))/10 if $attemptedSetHash{$sectionName};
	$item = sprintf '%.1f', $item;
	$dataRow .= "$item $DELIM";
	$dataRow .= "$total_possible";
	push @dataArray, $dataRow;
}
# print "<pre> @dataArray <\pre>";

print <<EOF;
<HR><BR>
<h3 align="left">Overall Scores by section. $all is the total for all included sections</h3>
EOF

## output the html table
my $string = &delimitedArray2htmlTable(\@dataArray, 'htmltable');
print "${string}<BR>";

## now output the info on individaul problems


### display whole course data
@dataArray =();
$dataRow = "$all $DELIM";
for ($i=1;$i<=$noOfProbs;$i++) {
	$dataRow .= "\# $i  $DELIM";
}
$dataRow =~ s|$DELIM$||;  ## remove last \$DELIM

push @dataArray, $dataRow;

$dataRow = "\% attempting $DELIM";
for ($i=1;$i<=$noOfProbs;$i++) {
	$item =0;   ## percent of students who have attempted this problem
	$item = int(100*$attemptedProbHash{$all}[$i]/$allStudentsHash{$all}+.5) if $allStudentsHash{$all};
	$dataRow .= "$item $DELIM";

}



$dataRow =~ s|$DELIM$||;  ## remove last \$DELIM
push @dataArray, $dataRow;



$dataRow = "percent correct $DELIM";  ## all sections and total # of students
for ($i=1;$i<=$noOfProbs;$i++) {
	$item =0;  ## % of students who have attempted this problem
	$item = int(100*$YHash{$all}[$i]/$attemptedProbHash{$all}[$i]+.5) if $attemptedProbHash{$all}[$i];
	$dataRow .= "$item $DELIM";
}
$dataRow =~ s|$DELIM$||;  ## remove last \$DELIM
push @dataArray, $dataRow;

$dataRow = "average incorrect $DELIM";  ## all sections and total # of students
for ($i=1;$i<=$noOfProbs;$i++) {

	$item =0;   ## avg # of incorrect attempts of students who have attempted this problem
	$item = (int(10*$NHash{$all}[$i]/$attemptedProbHash{$all}[$i]+.5))/10 if $attemptedProbHash{$all}[$i];
	$item = sprintf '%.1f', $item;
	$dataRow .= "$item $DELIM";
}
$dataRow =~ s|$DELIM$||;  ## remove last \$DELIM
push @dataArray, $dataRow;

print <<EOF;

<h3 align="left">Data on individual problems by section. $all is the total for all included sections</h3>
EOF
### output the html table
#$string = &delimitedArray2htmlTable(\@dataArray, 'htmltable');
#print "${string}<BR>";


## display data for sections

foreach $sectionName (@sectionNameArray) {

	## add blank row to table to seperate sections
	$dataRow = '<BR>';
	push @dataArray, $dataRow;
#    @dataArray =();
	$dataRow = "$sectionName $DELIM";
	for ($i=1;$i<=$noOfProbs;$i++) {
		$dataRow .= "\# $i  $DELIM";
	}
	$dataRow =~ s|$DELIM$||;  ## remove last \$DELIM

	push @dataArray, $dataRow;

	$dataRow = "\% attempting $DELIM";
	for ($i=1;$i<=$noOfProbs;$i++) {
		$item =0;   ## percent of students who have attempted this problem
		$item = int(100*$attemptedProbHash{$sectionName}[$i]/$allStudentsHash{$sectionName}+.5) if $allStudentsHash{$sectionName};
		$dataRow .= "$item $DELIM";
	}

$dataRow =~ s|$DELIM$||;  ## remove last \$DELIM
push @dataArray, $dataRow;




	$dataRow = "percent correct $DELIM";  ## all sections and total # of students
	for ($i=1;$i<=$noOfProbs;$i++) {
		$item =0;  ## % of students who have attempted this problem
		$item = int(100*$YHash{$sectionName}[$i]/$attemptedProbHash{$sectionName}[$i]+.5) if $attemptedProbHash{$sectionName}[$i];
		$dataRow .= "$item $DELIM";
	}
	$dataRow =~ s|$DELIM$||;  ## remove last \$DELIM
	push @dataArray, $dataRow;

	$dataRow = "average incorrect $DELIM";  ## all sections and total # of students
	for ($i=1;$i<=$noOfProbs;$i++) {

	   $item =0;   ## avg # of incorrect attempts of students who have attempted this problem
	   $item = (int(10*$NHash{$sectionName}[$i]/$attemptedProbHash{$sectionName}[$i]+.5))/10 if $attemptedProbHash{$sectionName}[$i];
	   $item = sprintf '%.1f', $item;
	   $dataRow .= "$item $DELIM";
	}
	$dataRow =~ s|$DELIM$||;  ## remove last \$DELIM
	push @dataArray, $dataRow;

#    ## output the html table
#    my $string = &delimitedArray2htmlTable(\@dataArray, 'htmltable');
#    print "${string}<BR>";

}

## output the html table
$string = &delimitedArray2htmlTable(\@dataArray, 'htmltable');
print "${string}<BR>";



## Now print everything again for excluded sections

##################################################################################################

print <<EOF;
<HR><HR><BR>

<h3 align="left">Current Statistics for Excluded Sections</h3>

Excluded sections typically contain practices uses, T.A.'s, prof's etc.  The list
of excluded sections in contained in the webworkCourse.ph file and typically consists
of a section with an empty name.
<p>
EOF



&initializeAndLoadUp(\@excludedSectionNameArray,'Excluded');
$all = 'Excluded';

## create  $dat arrays with the info. on average scores, etc


@dataArray =();

## first row

$dataRow = "sections$DELIM \# of students$DELIM \% attempting set $setNumber$DELIM average score$DELIM possible score";
push @dataArray, $dataRow;

## display whole course data

$dataRow = "$all $DELIM $allStudentsHash{$all} $DELIM";  ## all sections and total # of students

$item =0;  ## % of students who have attempted at least one problem in set
$item = int(100*$attemptedSetHash{$all}/$allStudentsHash{$all}+.5) if $allStudentsHash{$all};
$dataRow .= "$item $DELIM";

$item =0;  ## avg score of students who have attempted at least one problem in set
$item = (int($totalScoreHash{$all} *10/$attemptedSetHash{$all}+.5))/10 if $attemptedSetHash{$all};
$item = sprintf '%.1f', $item;
$dataRow .= "$item $DELIM";
$dataRow .= "$total_possible";

push @dataArray, $dataRow;


## display data for sections
foreach $sectionName (@excludedSectionNameArray) {
	$dataRow = "$sectionName $DELIM $allStudentsHash{$sectionName} $DELIM";  ## all sections and total # of students

	$item =0;  ## % of students who have attempted at least one problem in set
	$item = int(100*$attemptedSetHash{$sectionName}/$allStudentsHash{$sectionName}+.5) if $allStudentsHash{$sectionName};
	$dataRow .= "$item $DELIM";

	$item =0;  ## avg score of students who have attempted at least one problem in set
	$item = (int($totalScoreHash{$sectionName} *10/$attemptedSetHash{$sectionName}+.5))/10 if $attemptedSetHash{$sectionName};
	$item = sprintf '%.1f', $item;
	$dataRow .= "$item $DELIM";
	$dataRow .= "$total_possible";
	push @dataArray, $dataRow;
}
# print "<pre> @dataArray <\pre>";

print <<EOF;
<HR><BR>
<h3 align="left">Overall Scores by section. $all is the total for all excluded sections</h3>
EOF

## output the html table
$string = &delimitedArray2htmlTable(\@dataArray, 'htmltable');
print "${string}<BR>";

## now output the info on individaul problems


### display whole course data
@dataArray =();
$dataRow = "$all $DELIM";
for ($i=1;$i<=$noOfProbs;$i++) {
	$dataRow .= "\# $i  $DELIM";
}
$dataRow =~ s|$DELIM$||;  ## remove last \$DELIM

push @dataArray, $dataRow;

$dataRow = "\% attempting $DELIM";
for ($i=1;$i<=$noOfProbs;$i++) {
	$item =0;   ## percent of students who have attempted this problem
	$item = int(100*$attemptedProbHash{$all}[$i]/$allStudentsHash{$all}+.5) if $allStudentsHash{$all};
	$dataRow .= "$item $DELIM";

}



$dataRow =~ s|$DELIM$||;  ## remove last \$DELIM
push @dataArray, $dataRow;



$dataRow = "percent correct $DELIM";  ## all sections and total # of students
for ($i=1;$i<=$noOfProbs;$i++) {
	$item =0;  ## % of students who have attempted this problem
	$item = int(100*$YHash{$all}[$i]/$attemptedProbHash{$all}[$i]+.5) if $attemptedProbHash{$all}[$i];
	$dataRow .= "$item $DELIM";
}
$dataRow =~ s|$DELIM$||;  ## remove last \$DELIM
push @dataArray, $dataRow;

$dataRow = "average incorrect $DELIM";  ## all sections and total # of students
for ($i=1;$i<=$noOfProbs;$i++) {

	$item =0;   ## avg # of incorrect attempts of students who have attempted this problem
	$item = (int(10*$NHash{$all}[$i]/$attemptedProbHash{$all}[$i]+.5))/10 if $attemptedProbHash{$all}[$i];
	$item = sprintf '%.1f', $item;
	$dataRow .= "$item $DELIM";
}
$dataRow =~ s|$DELIM$||;  ## remove last \$DELIM
push @dataArray, $dataRow;

print <<EOF;

<h3 align="left">Data on individual problems by section. $all is the total for all excluded sections</h3>
EOF
### output the html table
#$string = &delimitedArray2htmlTable(\@dataArray, 'htmltable');
#print "${string}<BR>";


## display data for sections

foreach $sectionName (@excludedSectionNameArray) {

	## add blank row to table to seperate sections
	$dataRow = '<BR>';
	push @dataArray, $dataRow;
#    @dataArray =();
	$dataRow = "$sectionName $DELIM";
	for ($i=1;$i<=$noOfProbs;$i++) {
		$dataRow .= "\# $i  $DELIM";
	}
	$dataRow =~ s|$DELIM$||;  ## remove last \$DELIM

	push @dataArray, $dataRow;

	$dataRow = "\% attempting $DELIM";
	for ($i=1;$i<=$noOfProbs;$i++) {
		$item =0;   ## percent of students who have attempted this problem
		$item = int(100*$attemptedProbHash{$sectionName}[$i]/$allStudentsHash{$sectionName}+.5) if $allStudentsHash{$sectionName};
		$dataRow .= "$item $DELIM";
	}

$dataRow =~ s|$DELIM$||;  ## remove last \$DELIM
push @dataArray, $dataRow;




	$dataRow = "percent correct $DELIM";  ## all sections and total # of students
	for ($i=1;$i<=$noOfProbs;$i++) {
		$item =0;  ## % of students who have attempted this problem
		$item = int(100*$YHash{$sectionName}[$i]/$attemptedProbHash{$sectionName}[$i]+.5) if $attemptedProbHash{$sectionName}[$i];
		$dataRow .= "$item $DELIM";
	}
	$dataRow =~ s|$DELIM$||;  ## remove last \$DELIM
	push @dataArray, $dataRow;

	$dataRow = "average incorrect $DELIM";  ## all sections and total # of students
	for ($i=1;$i<=$noOfProbs;$i++) {

	   $item =0;   ## avg # of incorrect attempts of students who have attempted this problem
	   $item = (int(10*$NHash{$sectionName}[$i]/$attemptedProbHash{$sectionName}[$i]+.5))/10 if $attemptedProbHash{$sectionName}[$i];
	   $item = sprintf '%.1f', $item;
	   $dataRow .= "$item $DELIM";
	}
	$dataRow =~ s|$DELIM$||;  ## remove last \$DELIM
	push @dataArray, $dataRow;

#    ## output the html table
#    my $string = &delimitedArray2htmlTable(\@dataArray, 'htmltable');
#    print "${string}<BR>";

}

## output the html table
$string = &delimitedArray2htmlTable(\@dataArray, 'htmltable');
print "${string}<BR>";






####################################################################################################

print &htmlBOTTOM("profCourseData.pl", \%inputs, 'profCourseSecDataHelp.html');

# begin Timing code
my $endTime = new Benchmark;
&Global::logTimingInfo($beginTime,$endTime,"profCourseData.pl",$inputs{'course'},$inputs{'user'});
# end Timing code
exit;


##subroutines

sub processSectionNameArrays {
	## first consider all blank entries (i.e. empty or white space) in excludedSectionNameArray
	## to be the same section, namely ''
	my %temp_hash =();
	my $sectionName;
	foreach $sectionName (@excludedSectionNameArray) {
		if ((!defined ($sectionName)) or ($sectionName =~ m|^\s*$|)) {$sectionName = '';}
		$temp_hash{$sectionName} =1;
	}
	@excludedSectionNameArray = keys %temp_hash;

	## Make sure excludedSectionNameArray is a subset of sectionNameArray
	%temp_hash =();
	grep($temp_hash{$_}++,@sectionNameArray);
	@excludedSectionNameArray = grep($temp_hash{$_},@excludedSectionNameArray);

	## Now remove excluded sections from section list
	%temp_hash =();
	grep($temp_hash{$_}++,@excludedSectionNameArray);
	@sectionNameArray = grep(!$temp_hash{$_},@sectionNameArray);

	@sectionNameArray = sort @sectionNameArray;
	@excludedSectionNameArray = sort @excludedSectionNameArray;
}

sub initializeAndLoadUp {
	my ($sectionNameArrayref, $wholeCourseName) = @_;
	my @sectionNameArray = @$sectionNameArrayref;
	my $all = $wholeCourseName;
	my ($sectionName,$i,$score);

	## compute total possible points
	$total_possible = 0;
	for ($i=1;$i<=$noOfProbs;$i++) {$total_possible += $problemValueList[$i];}

	foreach $sectionName (@sectionNameArray) {
		$totalScoreHash{$sectionName} = 0;
		$attemptedSetHash{$sectionName} = 0;
		$allStudentsHash{$sectionName} = 0;
		for ($i=1;$i<=$noOfProbs;$i++) {
			$NHash{$sectionName}[$i]=0;   ## contains number of incorrect responses
			$YHash{$sectionName}[$i]=0;   ## contains totals of problem scores (i.e. a
										  ## number in [0,1] for each problem
			$attemptedProbHash{$sectionName}[$i]=0;
		}
	}

	## load up hashes with data

	my @pinNumbersArray = &getAllProbSetKeysForSet($setNumber);
	my ($studentAttemptedAProblem, $numIncorr, $pin, $status,$login_name_for_psvn);
		foreach $pin (@pinNumbersArray) {
			&attachProbSetRecord($pin);
			$login_name_for_psvn = &getStudentLogin($pin);
			attachCLRecord($login_name_for_psvn);
			$status = CL_getStudentStatus($login_name_for_psvn);
			next if dropStatus($status);
			$studentAttemptedAProblem = 0;
			$sectionName = &CL_getClassSection($login_name_for_psvn);
			if ((!defined ($sectionName)) or ($sectionName =~ m|^\s*$|)){
						$sectionName = '';
			}
			$allStudentsHash{$sectionName} += 1;

				for ($i=1;$i<=$noOfProbs;$i++) {
					if (&getProblemAttempted($i,$pin)) {
						$studentAttemptedAProblem = 1;
						$numIncorr = &getProblemNumOfIncorrectAns($i,$pin);
						$numIncorr =0 unless defined $numIncorr;
						$numIncorr = &min($numIncorr,99);
						$NHash{$sectionName}[$i] += $numIncorr;
						$attemptedProbHash{$sectionName}[$i] += 1;
						$score = &getProblemStatus($i,$pin);
						$YHash{$sectionName}[$i] += $score;
						$totalScoreHash{$sectionName} += round_score($score*$problemValueList[$i]);
					}
				}
				if ($studentAttemptedAProblem) {$attemptedSetHash{$sectionName} += 1;}
			}

	## initialize whole course data
		$totalScoreHash{$all} = 0;
		$attemptedSetHash{$all} = 0;
		$allStudentsHash{$all} = 0;
		for ($i=1;$i<=$noOfProbs;$i++) {
			$NHash{$all}[$i]=0;
			$YHash{$all}[$i]=0;
			$attemptedProbHash{$all}[$i]=0;
		}

	## compute whole course data
	foreach $sectionName (@sectionNameArray) {
		$totalScoreHash{$all} += $totalScoreHash{$sectionName};
		$attemptedSetHash{$all} += $attemptedSetHash{$sectionName};
		$allStudentsHash{$all} += $allStudentsHash{$sectionName};
		for ($i=1;$i<=$noOfProbs;$i++) {
			$NHash{$all}[$i] += $NHash{$sectionName}[$i];
			$YHash{$all}[$i] += $YHash{$sectionName}[$i];
			$attemptedProbHash{$all}[$i] += $attemptedProbHash{$sectionName}[$i];
		}
	}
}
## find unique name
sub uniqueCourseName {
	my $all = $classID;
	if (exists $allStudentsHash{$all}) {
		my $j = 1;
		$all = "${all}$j";
		while (exists $allStudentsHash{$all}) {
			$j++;
			$all = "${all}$j";
		}
	}
$all
}



sub warningMsgPage {

my ($inputref,$warningMsg)  = @_;
my %inputs = %$inputref;
# print HTML text
print &htmlTOP("Scoring Utilities");

# print navigation buttons
print qq!
<A HREF="${Global::cgiWebworkURL}profLogin.pl?user=$inputs{'user'}&key=$inputs{'key'}&course=$inputs{'course'}">
<IMG SRC="${Global::upImgUrl}" align="right" BORDER=1 ALT="[Up]"></A><p>
!;

print <<EOF;
<HR><BR>
<h3 align="left">WeBWorK scoring WARNING message for $classID:</h3>
EOF
print "<pre>";
print $warningMsg;
print "</pre>";
print "To Continue displaying data for $setNumber, check the \"Continue\" check box and then select
\"Continue displaying data for $setNumber\". <BR> To Quit and return to the Scoring page, select the \"Up\"
buttom.";

## profScoring form -- allows you to score problem sets
print <<EOF;
<P>

		<FORM METHOD = "POST" ACTION= "${cgiURL}profCourseData.pl">
		 <INPUT TYPE="SUBMIT" VALUE="Continue displaying data for $setNumber">

EOF


	print qq{
		<BR><INPUT TYPE=CHECKBOX NAME="batchMode" VALUE=1 >Continue
		};

# resume printing the rest of the form

print <<EOF;
		 <INPUT TYPE="HIDDEN" NAME="user" VALUE="$inputs{'user'}">
		 <INPUT TYPE="HIDDEN" NAME="key" VALUE="$inputs{'key'}">
		 <INPUT TYPE="HIDDEN" NAME="course" VALUE="$inputs{'course'}">
		 <INPUT TYPE="HIDDEN" NAME="setNo" VALUE="$inputs{'setNo'}">
		 </FORM>
EOF


print &htmlBOTTOM("profLogin.pl", \%inputs);
exit;
} #end of warning Page






