#!/usr/local/bin/webwork-perl

## This file is profEditSetDef.pl
## It provides a utility for professors to edit set definition files
## and their course's webworkCourse.ph

use lib '.'; use webworkInit; # WeBWorKInitLine

use Global;
use CGI qw(:standard);
use Auth;
use Net::SMTP;
use HTML::Entities;
use strict;

my $cgi = new CGI;
my %inputs = $cgi->Vars();

#for use in strings where $cgi->param is not interpolated correctly or at all
my $course = $cgi->param('course');
my $user = $cgi->param('user');
my $session_key = $cgi->param('key');

my $currentFile = $cgi->param('filename');
my $currentDir = $cgi->param('dir'); # relative to extDir below
$currentDir ='' unless defined $currentDir;
my $currentExt = $cgi->param('ext'); # this is the expected extension

# edit these to add functionality for other file types
my %validExt = ('def' => 1, 			
		'ph' => 1,
		'pg' => 1,
		'html' => 1);

my %extName = ( 'def' => "Set Definition",
		'ph' => "Course Environment",
		'pg' => "Set Header",
		'html' => "Course index.html");

my %extDir = (  'def' => "/templates/",
		'ph' => "/",
		'pg' => "/templates/",
		'html' => "/html/");

&Global::getCourseEnvironment($course);

my $scriptDirectory		= $Global::scriptDirectory;
my $databaseDirectory	  	= $Global::databaseDirectory;
my $htmlURL			= $Global::htmlURL;
my $cgiURL			= $Global::cgiWebworkURL;
my $courseDirectory		= $Global::coursesDirectory . $course;
my $courseScriptsDirectory 	= $Global::courseScriptsDirectory;
my $templateDirectory	  	= getCourseTemplateDirectory;
my $feedbackAddress		= $Global::feedbackAddress;

require "${scriptDirectory}$Global::FILE_pl";
require "${scriptDirectory}$Global::classlist_DBglue_pl";
require "${scriptDirectory}$Global::HTMLglue_pl";
require "${scriptDirectory}$Global::FILE_pl";

# log access
&Global::log_info('', query_string);

my $permissionsFile = &Global::getCoursePermissionsFile($course);
my $permissions = &get_permissions($user, $permissionsFile);
my $keyFile = &Global::getCourseKeyFile($course);


#verify session key	
&verify_key($user, $session_key, "$keyFile", $course);

#verify permissions are correct
if (($permissions != $Global::instructor_permissions) and ($permissions != $Global::TA_permissions) ) {
	print "permissions = $permissions instructor_permissions = $Global::instructor_permissions\n";
	print &html_NO_PERMISSION;
	exit(0);
	}

# verify that the file type is valid
if (!($currentFile =~ /.*\.(.*)/) || ($1 ne $currentExt) ) {
	&user_error("The file $currentFile is not valid because it does not contain the expected ending: $currentExt");
}

# consolidate relative directory paths into one hard path
$currentDir = $courseDirectory . $extDir{$currentExt} . $currentDir;

#wwerror("current Dir: $currentDir<P>current File:$currentFile <P>current Ext: $currentExt");
# make sure requested file exists and is a valid file
	if (defined($currentFile) && -e "$currentDir$currentFile") {
		unless ( -R "$currentDir$currentFile") {
			wwerror ('File is not readable', "The file $currentFile is not readable by the webserver. Check that it's permissions are set correctly.");
		}
	} else {
		wwerror('File does not exist', "The file $currentDir$currentFile does appear to exist, please check the filename and try again.");
	}	

# get row and column info if submitted
	my $rows = (defined($cgi->param('rows'))) ? $cgi->param('rows') : $Global::editor_window_rows;
	my $columns = (defined($cgi->param('columns'))) ? $cgi->param('columns') : $Global::editor_window_columns;

# Deal with filled out forms and various actions resulting from different buttons
	if ( defined($cgi->param('action')) ) {
		if (defined($cgi->param('savefilename'))) {
			&user_error("For security reasons, you cannot save a file in any directory higher than the current directory.  Please specify a file name to save under.") if ($cgi->param('savefilename') =~ /^[~.]/ || $cgi->param('savefilename') =~ /\.\./);
		}

		#if Save button was clicked
		if (( $cgi->param('action') eq 'Save') && defined($cgi->param('body')) && defined($cgi->param('savefilename'))) {

			my $temp_body = $cgi->param('body');
			$temp_body =~ s/\r\n/\n/g; #html textareas tend to leave \r

#			saveProblem($temp_body, $currentDir . $cgi->param('savefilename'));
			saveProblem($temp_body, $cgi->param('savefilename'));
			$currentFile = $cgi->param('savefilename');

		#if Save As button was clicked
		} elsif (( $cgi->param('action') eq 'Save as') && defined($cgi->param('body')) && defined($cgi->param('savefilename'))) {

			$currentFile = $cgi->param('savefilename');

#			if ($currentFile =~ /^[~.]/ || $currentFile =~ /\.\./) {
#				&user_error("For security reasons, you cannot specify a merge file from a directory higher than the email directory (you can't use ../blah/blah).  Please specify a different file or move the needed file to the email directory");
#			}


			my $temp_body = $cgi->param('body');
			$temp_body =~ s/\r\n/\n/g;

			saveNewProblem($temp_body, $currentFile);

		}
	}



#Begin Page
	print &htmlTOP("Edit $extName{$currentExt} File(s) for $course"),
		$cgi->a( { -href=>"${cgiURL}login.pl?user=$user&key=$session_key&course=$course" },

			$cgi->img({ -name=>'upImg',
					  -src=>"${Global::upImgUrl}",
					  -align=>'right',
					  -border=>'1',
					  -alt=>'[Up]'
					})
			),
	$cgi->p;

	print "\n",
	 $cgi->hr, $cgi->br,
	 "\n\n", $cgi->h3({ -align=>'left' }, "WeBWorK $extName{$currentExt} File(s) Editor for $course"), "\n",
	 $cgi->p,
	 "From this page, you can edit your $extName{$currentExt} file(s).  You can both save over
	 old files and create new ones.  You can even open one file, make changes and then save those
	 changes under a different file name.",
	 $cgi->hr, "\n";


# start form with hidden entries to pass info back to profEditCourseFiles.pl
# the last two are very circular but are necessary to make sure the script
# continues to know which kind of file it is dealing with
# They should be set by the original call (most likely in profLogin.pl
	print $cgi->startform(-action=>"${cgiURL}profEditCourseFiles.pl"), "\n",
		 $cgi->hidden(-name=>"user", -value=>$user), "\n",
		 $cgi->hidden(-name=>"key", -value=>$session_key), "\n",
		 $cgi->hidden(-name=>"course", -value=>$course), "\n",

		 $cgi->hidden(-name=>"dir", -value=>$cgi->param("dir")), "\n",
		 $cgi->hidden(-name=>"ext", -value=>$cgi->param("ext")), "\n"; 

#get current file
	my ($text, @text);
	if (-e "$currentDir$currentFile") {
	 	open FILE, "$currentDir$currentFile";
		@text = <FILE>;
		$text = join "", @text;
	} else {
		# wwerror($0, "Message File $currentDir$currentFile does not exist!");
	}

#print actual body of message
	print "\n", $cgi->br, $cgi->textarea(-name=>'body', -default=>$text, -rows=>$rows, -columns=>$columns, -override=>1);


#get all message files and create a list
# 
# 	@filenames = sort @filenames;
	
	my @filenames;
	
	if($currentExt eq "pg") {
		@filenames = grep /\.*header\.*.pg$/i, recursiveFindFiles($currentDir);
		map s|$templateDirectory/?||, @filenames;
	} else {
		opendir CURDIR, $currentDir; # or wwerror($0, "Can't open directory $currentDir","","");
		@filenames = grep /\.$currentExt$/, readdir CURDIR;
		closedir CURDIR;
	}
	@filenames = sort @filenames;

	print $cgi->p, $cgi->popup_menu(-name=>'filename', -values=>\@filenames, -default=>$currentFile);


#create all necessary action buttons
	print $cgi->submit(-name=>'action', -value=>'Open'), "\n", $cgi->br,
		 $cgi->textfield(-name=>'savefilename', -size => 20, -value=> "$currentFile", -override=>1), ' ',
		 $cgi->submit(-name=>'action', -value=>'Save'), " \n",
		 $cgi->submit(-name=>'action', -value=>'Save as'), " \n",
		 'For "Save As" choose a new filename.', $cgi->br, $cgi->br,
		 $cgi->p, $cgi->submit(-name=>'action', -value=>'Revert to original and Resize message window'),
		 " Rows: ", $cgi->textfield(-name=>'rows', -size=>3, -value=>$rows),
		 " Columns: ", $cgi->textfield(-name=>'columns', -size=>3, -value=>$columns),
		 $cgi->br, "If you resize the message window, you will lose all unsaved changes and the students you have selected may be unselected.",
		 $cgi->end_form, 
		 &htmlBOTTOM("profEditCourseFiles", \%inputs);

# End of HTML


###### SUBROUTINES ######
sub saveProblem {
	my ($body, $probFileName)= @_;

	open (PROBLEM, ">$currentDir$probFileName") ||
		wwerror($0, "Could not open $currentDir$probFileName for writing.
		Check that the  permissions for this problem are 660 (-rw-rw----)");
	print PROBLEM $body;
	close PROBLEM;
	chmod 0660, "$currentDir$probFileName" ||
	             print "Content-type: text/html\n\n
	                    CAN'T CHANGE PERMISSIONS ON FILE $currentDir$probFileName";

}

sub saveNewProblem {
	my ($body, $new_file_name)= @_;
 #######check that the new file name doesn't exist
	if (-e "$currentDir$new_file_name" ) {
		wwerror("Can not use this file name", "The file\n".
		"$new_file_name\n".
		"already exists.\n" .
		"<b>The new version was not saved.</b>\n" .
		"Go back and choose a different file name or\, if you really want to edit\n".
		"$new_file_name\,\n".
		"go back and hit the \&quot;Save updated version\&quot; button.");
	}

	wwerror ("Invalid file name", "The file name \"$new_file_name\" does not have a \".$currentExt\" extension.
<b>The file was not saved.</b>
Go back and choose a file name with a \".$currentExt\" extension.") unless
		$new_file_name =~ m|\.$currentExt$|;
 #######copy new version to the file new_file_name
	open (PROBLEM, ">$currentDir$new_file_name") ||
		wwerror($0, "Could not open $currentDir$new_file_name for writing.
		Check that the  permissions for the directory $currentDir are 770 (drwxrwx---)");
	print PROBLEM $body;
	close PROBLEM;
	chmod 0660, "$currentDir$new_file_name" ||
	             print "Content-type: text/html\n\n
	                    CAN'T CHANGE PERMISSIONS ON FILE $currentDir$new_file_name";

}

sub user_error {
    my $msg = join " ", @_;
    print $cgi->header,
	$cgi->start_html('-title' => 'User error'),
	$cgi->h1('User error'),
	$cgi->p,
	$cgi->b(HTML::Entities::encode($msg)),
	$cgi->p,
        "Please hit the &quot;<B>Back</B>&quot; button on your browser to ",
	"try again, or notify ", $cgi->br,
	"&lt;", $cgi->a({href=>"mailto:$Global::webmaster"}, $Global::webmaster), "&gt; ",
	"if you believe this message is in error.",
	$cgi->end_html;
    exit(1);
}


sub recursiveFindFiles
{
	my $dir = shift;
	$dir .= '/' unless $dir =~ m|/$|;
	opendir DIR, $dir;
	my @items = readdir DIR;
	closedir DIR;
	my @result;
	foreach (@items) {
		/^\./ and next;
		-f "$dir$_" and push @result, "$dir$_";
		-d "$dir$_" and push @result, recursiveFindFiles("$dir$_");
	}
	return @result;
}
