#!/usr/local/bin/webwork-perl

## This file is profExportClasslistDatabase.pl
##

####################################################################
# Copyright @ 1995-2000 University of Rochester
# All Rights Reserved
####################################################################

use lib '.'; use webworkInit; # WeBWorKInitLine
use CGI qw(:standard);
use Global;
use Auth;
use strict;

my $cgi = new CGI;
my %inputs = $cgi->Vars();

# get information from CGI inputs  (see also below for additional information)

my $Course 			= $inputs{'course'};
my $User 			= $inputs{'user'};
my $Session_key 	= $inputs{'key'};

# verify that information has been received
	unless($Course && $User && $Session_key) {
		&wwerror("$0","The script did not receive the proper input data.","","");
		}

# establish environment for this script

&Global::getCourseEnvironment($inputs{'course'});


my $scriptsDirectory 	= getWebworkScriptDirectory;
my $databaseDirectory 		= getCourseDatabaseDirectory;
my $templateDirectory       = getCourseTemplateDirectory;
my $cgiURL   				= getWebworkCgiURL;

# File names

require "${scriptsDirectory}$Global::HTMLglue_pl";
require "${scriptsDirectory}$Global::classlist_DBglue_pl";
require "${scriptsDirectory}$Global::FILE_pl";;

# log access
	&Global::log_info('', query_string);


my $permissionsFile = &Global::getCoursePermissionsFile($inputs{'course'});
my $permissions = &get_permissions($inputs{'user'}, $permissionsFile);
my $keyFile = &Global::getCourseKeyFile($inputs{'course'});

#verify session key
&verify_key($inputs{'user'}, $inputs{'key'}, $keyFile, $inputs{'course'});

# verify permissions are correct
if ($permissions != $Global::instructor_permissions ) {
	print "permissions = $permissions instructor_permissions = $Global::instructor_permissions\n";
	print &html_NO_PERMISSION;
	exit(0);
}
# get the rest of the information from the submitted form

my $outFileName = $inputs{'savefilename'};
my $overWrite = $inputs{'overWrite'};
my $backUp = $inputs{'backUp'};  

$overWrite = 0 unless defined $overWrite;
$backUp = 0 unless defined $backUp;

my $CL_status = get_CL_database_status();
wwerror('Classlist Database is unlocked', 'You must go back and lock the classlist database
before you can export it to an ascii file.') unless $CL_status eq 'locked';

if ($outFileName =~ /([^\/\\]+)$/) {
	$outFileName = $1;	
	$outFileName =~ s/^\.+//;		
} 
else {
	wwerror($0, "The fileName $outFileName is invalid. Go back and type in another file name.");
}

$outFileName =~  s|\..*||;      ## remove extension 
$outFileName = "${outFileName}\.lst";      ## make sure filename has .lst extension
my $fullPathName = "${templateDirectory}$outFileName";
if ((-e "$fullPathName") and (not $overWrite)) {    
	&overWriteError();
    exit;
}    

if ((-e "$fullPathName") and  $backUp) 	{&backupFile("$fullPathName");}   ##make up to three backups

my ($studentID, $lastName, $firstName, $status, $comment,  $section, $recitation, $email_address, $login_name);
my @login_names = @{&getAllLoginNamesSortedBySectionThenByName()};

open(OUTFILE,">$fullPathName") or wwerror("$0","Can't open $fullPathName for writing.");

foreach $login_name (@login_names) {

	attachCLRecord($login_name);

	$studentID =     &CL_getStudentID           ($login_name);
	$lastName =      &CL_getStudentLastName     ($login_name);
	$firstName =     &CL_getStudentFirstName    ($login_name);
	$status =        &CL_getStudentStatus       ($login_name);
	$comment =       &CL_getComment             ($login_name);
	$section =       &CL_getClassSection        ($login_name);
	$recitation =    &CL_getClassRecitation     ($login_name);
	$email_address = &CL_getStudentEmailAddress ($login_name);


    print OUTFILE "$studentID\, $lastName\, $firstName\, $status\, $comment\, $section\, $recitation\,  $email_address\, $login_name \n";


}

close(OUTFILE);

&columnPrint("$fullPathName","$fullPathName");
chmod($Global::classlist_file_permission, "$fullPathName") or
  &wwerror("$0","Can't do chmod($Global::classlist_file_permission, $fullPathName)");
chown(-1,$Global::numericalGroupID,"$fullPathName") or
  &wwerror("$0","Can't do chown(-1,$Global::numericalGroupID,$fullPathName)");

uploadSuccess("$fullPathName");
exit;

sub uploadSuccess {
  my ($savedFile) = @_;
  print"content-type: text/html\n\n<H2>Success, file saved. </H2>\n";
  print "The classlist database has been exported to the file:<BR>$savedFile";    

  print &htmlBOTTOM("profExportClasslistDatabase.pl", \%inputs);
}


sub overWriteError {
    print"content-type: text/html\n\n<H2>No changes made:  You are not in overwrite mode</H2>\n";
    print "The file $fullPathName already exists.  If you want to overwrite this file, 
    use your browser's Back Button to return to the previous page and check the overwrite check box.
    Or you can go back and type in a different filename.";
}