#!/usr/local/bin/webwork-perl

## This file is profNewPassword.pl
## It allows a prof to give a user a newpassword
##

require 5.001;
use strict;
use lib '.'; use webworkInit; # WeBWorKInitLine

use Global;
use Auth;
use GDBM_File;
use CGI qw(:standard);

my $User    = param('user')    || &error_form('No user', 'No user name specified.'); 
my $Key     = param('key')     || &error_form('No key', 'No session key specified.'); 
my $Course  = param('course')  || &error_form('No course', 'No course specified.');
my $studentLogin	= param('studentLogin')|| &error_form('No student selected', 'No student specified.');

my $Passwd  = param('passwd');
my $Check   = param('check');

# remove any leading/trailing spaces in input
#$Passwd    =~ s|^\s*(.*?)\s*$|$1|;
#$Oldpasswd =~ s|^\s*(.*?)\s*$|$1|;
#$Check     =~ s|^\s*(.*?)\s*$|$1|;

&Global::getCourseEnvironment($Course);

my $scriptDirectory   = $Global::scriptDirectory;
require "${scriptDirectory}$Global::HTMLglue_pl";
require "${scriptDirectory}$Global::classlist_DBglue_pl"; 


my $Passwd_file = &getCoursePasswordFile($Course);
&Global::error("", "Can't read $Passwd_file") unless (-r $Passwd_file);

my $Key_file    = &getCourseKeyFile($Course);
&Global::error("", "Can't read $Key_file") unless (-r $Key_file);


## make sure user's already been authenticated
&verify_key($User, $Key, $Key_file, $Course);

my $permissionsFile = &Global::getCoursePermissionsFile($Course);
my $permissions = &get_permissions($User, $permissionsFile);

if ($permissions != $Global::instructor_permissions ) {
	print "permissions = $permissions instructor_permissions = $Global::instructor_permissions\n";
	print &html_NO_PERMISSION;
	}

attachCLRecord($studentLogin);											
my $lastName		= CL_getStudentLastName($studentLogin);
my $firstName		= CL_getStudentFirstName($studentLogin);
my $studentID		= CL_getStudentID($studentLogin);


## if all the data's been entered, check it
if ($Passwd && $Check) {

    ## make sure they know what they entered
    if ($Passwd ne $Check) {
	&error_form("Password Mismatch", "Your passwords don't match.");
    }

    ## all's well - set a new password
    &new_password($studentLogin, $Passwd, $Passwd_file);
    &success_form;
}

## otherwise print the password change form (again, if necessary)
else {
    &change_pw_form;
    exit;
}


sub change_pw_form {
    my $course = shift;

    print &htmlTOP('New Password Page', $Global::background_plain_url),
	  hr,
	  h1('New Password Page'),
	  start_form('POST', url),   
	  "Choose a new password for ",b($firstName), " ", b($lastName)," ($studentID), $Course",
	  " account, ", b($studentLogin), ".",	   
	  p,
	  password_field('passwd', '', 18), " New password", br,
	  password_field('check', '', 18), " New password (again)", br,
	  p,
	  submit('Set the new password'),
	  hidden('studentLogin'),
	  hidden('user'),
  	  hidden('key'),
  	  hidden('course'),
	  end_form, end_html;
}


sub success_form {
    print &htmlTOP('Password Changed', $Global::background_okay_url),
	  hr,
	  h1('Password Changed'),
	  "The new password for ", b($firstName), " ", b($lastName)," ($studentID), $Course",
	  " account, ", b($studentLogin), " is in effect.",
	  p,
	  start_form('POST', "${Global::cgiWebworkURL}profLogin.pl"),
	  p,"\n",
	  hidden('course'), "\n", hidden('user'), "\n", hidden('key'),
	  submit('Continue'),
	  end_form, end_html;
}

sub error_form {
    my ($title, $msg) = @_;
    
    print &htmlTOP($title, $Global::background_warn_url),
	  hr,
	  h1($title),
	  $msg,
	  p,
	  "Hit the <B>Back</B> button and try again.",
	  end_form, end_html;
	  exit;
}
