#!/usr/local/bin/webwork-perl

## This file is profUploadClasslistFile.pl
## It provides access to utilities for uploading classlist files
##

####################################################################
# Copyright @ 1995-1998 University of Rochester
# All Rights Reserved
####################################################################

use lib '.'; use webworkInit; # WeBWorKInitLine
use CGI qw(:standard);
use File::Copy;
use Global;
use Auth;
use strict;

my $query =new CGI;
my %inputs = $query->Vars();
# get information from CGI inputs  (see also below for additional information)

my $Course 			= $query->param('course');
my $User 			= $query->param('user');
my $Session_key 	= $query->param('key');

my @array = %inputs;
# print "inputs are @array";

# verify that information has been received
unless($Course && $User && $Session_key) {
    &wwerror("$0","The script did not receive the proper input data.","","");
}




# establish environment for this script

&Global::getCourseEnvironment($inputs{'course'});


my $cgiURL   				= getWebworkCgiURL;
my $databaseDirectory 		= getCourseDatabaseDirectory;
my $htmlURL       			= getCourseHtmlURL;
my $scriptDirectory   		= getWebworkScriptDirectory;
my $templateDirectory 		= getCourseTemplateDirectory;
my $DAT                     = getDat;
my $dd                      = getDirDelim;

require "${scriptDirectory}$Global::DBglue_pl";
require "${scriptDirectory}$Global::FILE_pl";
require "${scriptDirectory}$Global::HTMLglue_pl";

# log access
&Global::log_info('', query_string);


my $permissionsFile = &Global::getCoursePermissionsFile($inputs{'course'});
my $permissions = &get_permissions($inputs{'user'}, $permissionsFile);
my $keyFile = &Global::getCourseKeyFile($inputs{'course'});

#verify session key
&verify_key($inputs{'user'}, $inputs{'key'}, $keyFile, $inputs{'course'});

# verify permissions are correct
if ($permissions != $Global::instructor_permissions ) {
    print "permissions = $permissions instructor_permissions = $Global::instructor_permissions\n";
    print &html_NO_PERMISSION;
    exit(0);
}
# get the rest of the information from the submitted form


my $overWrite     = $inputs{'overWrite'};
$overWrite = 0 unless defined $overWrite;
my $format        = $inputs{'format'};
my $backUp        = $inputs{'backUp'};
$backUp = 0 unless defined $backUp;
my $fileHandle    = $query->upload('upload');
my $fileName;
		
if ($fileHandle =~ /([^\/\\]+)$/) {
	$fileName = $1;	
	$fileName =~ s/^\.+//;		
} 
else {
	wwerror($0, "The fileName $fileHandle is invalid.");
}

my $ext = 'lst';
my $ext1 = 'html';
if ($format eq 'tab') {$ext ='txt';}
elsif ($format eq 'html') {$ext ='htm';}

if ($format eq 'csv') {	  		
    unless (($fileName =~ /\.lst$/) or ($fileName =~ /\.csv$/)) {	
	    wwerror($0, "The fileName $fileHandle is invalid. Only files with fileNames ending in .lst or .csv can be uploaded");
    }
} 

elsif ($format eq 'tab') {	  		
    unless ($fileName =~ /\.$ext$/){	
	    wwerror($0, "The fileName $fileHandle is invalid. Only files with fileNames ending in .txt can be uploaded");
    }
}
   
else {    
    unless (($fileName =~ /\.$ext$/) or ($fileName =~ /.$ext1$/)){	
	    wwerror($0, "The fileName $fileHandle is invalid. Only files with fileNames ending in .$ext or in 
	    .$ext1 can be uploaded");
    }
}

## get correct filename
my $shortFileName = $fileName;
$shortFileName      =~  s|\..*||;      ## remove extension 
my $saveFileName = "${shortFileName}\.lst";      ## make sure filename had lst extension

## check for overwrite mode
if ((-e "${templateDirectory}$saveFileName") and ($overWrite != 1)) {
    &overWriteError();
    exit;
}


    ## check for validity of format before saving file    
my $i=1;
while(-e "${templateDirectory}temp$i.lst") {$i++;}      ##don't overwrite existing files
my $tempFileName ="${templateDirectory}temp$i.lst";
			
open(OUTFILE, ">$tempFileName") or wwerror ($0, "Can't open $tempFileName File for Writing."); 

## read the file and save it to a temporary file.
## use bare block so that we can use  symbolic ref's under strict
    {  
    no strict 'refs';
    $| = 1;       
    my $Buffer;
    undef $Buffer;		
    while (read($fileHandle,$Buffer,1024)) {
        print OUTFILE $Buffer;
    }    		
    close($fileHandle);
    close(OUTFILE);
}    


## convert file to comma delimited form if necessary
if ($format eq 'tab') {
    open(INFILE, "$tempFileName") or wwerror ($0, "Can't open $tempFileName File for Reading."); 
    my @stringArray = <INFILE>;
    close(INFILE);
    my $string = join '', @stringArray;

    #convert newlines of original machine to newline of webwork machine
    $string =~ s/\015?\012|\015/\n/g;

    $string =~ s|\t|,|g; 
    $string =~ s|_TAB_|\t|g;
    open(OUTFILE, ">$tempFileName") or wwerror ($0, "Can't open $tempFileName File for Writing."); 
    print OUTFILE $string;
    close(OUTFILE);
}
elsif ($format eq 'html') {
    open(INFILE, "$tempFileName") or wwerror ($0, "Can't open $tempFileName File for Reading."); 
    my @stringArray = <INFILE>;
    close(INFILE);
    my $string = join '', @stringArray;

    #convert newlines of original machine to newline of webwork machine
    $string =~ s/\015?\012|\015/\n/g;

    my $workingString = &htmlPage2htmlTable($string);
    my $outString = &htmlTable2delim($workingString);
    open(OUTFILE, ">$tempFileName") or wwerror ($0, "Can't open $tempFileName File for Writing."); 
    print OUTFILE $outString;
    close(OUTFILE);
}
	
my $msg = htmlCheckClasslistFile($Global::noOfFieldsInClasslist,$tempFileName);
            ## remove temp file
if ($msg eq 'OK') { 
    if ($backUp and (-e "${templateDirectory}$saveFileName")) {        
        &backupFile("${templateDirectory}$saveFileName");   ##make up to three backups
    }    
    copy("$tempFileName" , "${templateDirectory}$saveFileName") or &wwerror($0, "Could not write to the file
      ${templateDirectory}$saveFileName .  The uploaded file has been saved as $tempFileName");
	chmod($Global::classlist_file_permission, "${templateDirectory}$saveFileName") or
      &wwerror("$0","Can't do chmod($Global::classlist_file_permission, ${templateDirectory}$saveFileName)","","", query_string());
    chown(-1,$Global::numericalGroupID,"${templateDirectory}$saveFileName") or
      &wwerror("$0","Can't do chown(-1,$Global::numericalGroupID,${templateDirectory}$saveFileName)","","", query_string());
 
    unlink $tempFileName or &wwerror($0, "Could not remove the file $tempFileName.");
    &uploadSuccess("${templateDirectory}$saveFileName");
}
else {
    $msg .= "<BR>GO BACK AND CORRECT YOUR FILE $fileHandle. <BR><strong>FILE NOT SAVED.</strong>";
    unlink $tempFileName or &wwerror($0, "Could not remove the file $tempFileName. <BR> Also $msg");
    &wwerror("$0","$msg","","", query_string());
}

sub overWriteError {
    print"content-type: text/html\n\n<H2>No changes made:  You are not in overwrite mode</H2>\n";
    print "The file ${templateDirectory}$saveFileName already exists.  If you want to overwrite this file, 
      use your browser's BacK Button to return to the previous page and check the overwrite check box or<BR>";
    print "<FORM METHOD=POST ACTION=\"${cgiURL}profClasslist.pl\"><P>";
    print &sessionKeyInputs(\%inputs);
    print <<"ENDOFHTML";
    <INPUT TYPE=SUBMIT VALUE="Return to Classlist Page">
    </FORM>
ENDOFHTML
    print &htmlBOTTOM("profUploadClasslistFile.pl", \%inputs);
}

sub uploadSuccess {
  my ($savedFile) = @_;
  print"content-type: text/html\n\n<H2>Success, file saved. </H2>\n";
  print "Your file has been uploaded to the file:<BR>$savedFile";    
  print "<FORM METHOD=POST ACTION=\"${cgiURL}profClasslist.pl\"><P>";
  print &sessionKeyInputs(\%inputs);
print <<"ENDOFHTML";
<INPUT TYPE=SUBMIT VALUE="Return to Classlist Page">
</FORM>
ENDOFHTML
  print &htmlBOTTOM("profUploadClasslistFile.pl", \%inputs);
}

