#!/usr/local/bin/webwork-perl

## $Id: pScSet6.pl,v 1.6 2001/09/06 15:48:35 apizer Exp $

####################################################################
# Copyright @ 1995-1998 University of Rochester
# All Rights Reserved
####################################################################


use strict;
# This file is pScSet6.pl
#

# For these scoring subroutines to work, the calling script must require
# the following files, i.e. it must contain the lines:
# require "webwork.ph";
# require "${scriptDirectory}$DBglue_pl";
# require "${scriptDirectory}$Global::classlist_DBglue_pl";
# require "${scriptDirectory}FILE.pl";

my $debugOn=0;

# Things defined in Global.pl (can be over ridden by statements in webworkCourse.ph)

my $scoringDirectory  = &getCourseScoringDirectory();
my $databaseDirectory = getCourseDatabaseDirectory();
my $DELIM = &getDelim();
my $scoreFilePrefix = &getScoreFilePrefix();
my $scoring_log = &getScoring_log();
my $dash = &getDash();
my $DAT = &getDat();
my @DBEXT = &getBbext();
my @STATUS_DROP = &getStatusDrop();
my %sortaaArray = ();  ## declared here so that we can use it in the sort routines

##Global Definition of headerline names
my @HL=("PIN","NO OF FIELDS","SET NUMBER","PROB NAME","DUE DATE","DUE TIME","PROB VALUE","STUDENT ID");


# &scoreMessage("scoring dir is $scoringDirectory");


sub scoreMessage {

	## Prints message to scoring log and STDERR
	my ($string) = $_[0];
	unless (-e "${scoringDirectory}${scoring_log}") {
	    &createFile("${scoringDirectory}${scoring_log}", $Global::scoring_files_permission, $Global::numericalGroupID);
	}
    open(LOGFILE, ">>${scoringDirectory}${scoring_log}")  or
      &wwerror("$0","Can't append to ${scoringDirectory}${scoring_log}");
	print LOGFILE "$string\n";
	print STDERR  "$string\n";
	close(LOGFILE);
}

sub get_raw_scores  {

	## Called with parameters
	## ($method, $setNumber, $dueDate, $dueTime, $problemListref, $problemValueListref)
	## The $method is either 'db' (the default) or 'scofiles' which
	## determined if the data will be read from the database or sco files
	## The last two parameters are only used with the 'scofiles' scoring method.
    ## Before this subroutine is run, the set definition file must be read with readSetDefinition

    ## get the number of Y's and N's from database or sco files S{set#}-{pin#}.sco and
    ## store them in three files: (1) s{set#}bak{#}.${DAT} by rows:  pin# ,ss# , lastname
    ##, firstnames , #Y , #N , #Y , #N ... .  Here bak{#} is bak1, bak2, etc.  This file is
    ## never overwritten.  Everytime the set is scored, a new bak file is created.
    ## (2) s{set#}ful.${DAT} by rows:  ss# , lastname , firstnames , #Y ,
    ## #N , #Y , #N ...  (3) s{set#}scr.${DAT} by rows:  ss# , lastname , firstnames
    ## , [01] , [01] , ... where 1 indicates a correct answer and 0 otherwise.  Files
    ## (2) and (3) have header rows and are suitable for analyzing with a spreadsheet or
    ## database program.  Note: the delimiters, shown above as comas (","), are set by
    ## $DELIM e.g.  $DELIM=","

	my ($method, $setNumber, $dueDate, $dueTime, $problemListref, $problemValueListref) = @_;
    my ($noOfProbs,$pin,$num,$warnTag,$i,$string,$dataLine,$probValue);
    my(@bakArray,@fulArray,@scrArray,@Yarray,@Narray,@Sarray,@temp,@pinNumbersArray);
    my(%assocArray);
    my(@probList,@probVal,$bakFileName);

    @pinNumbersArray = &getAllProbSetKeysForSet($setNumber);

	if ($method eq 'scofiles') {
    	 @probList = @$problemListref;
         @probVal = @$problemValueListref;
	}
	else {
		$pin =$pinNumbersArray[0];
		&attachProbSetRecord($pin) or &wwerror("$0","no Record $pin");
		@probList = &getAllProblemsForProbSetRecord($pin);
		@probList = sort( { $a <=> $b }  @probList);
		@probVal = ();
    	foreach $num (@probList) {
			$probValue = &getProblemValue($num,$pin);
			push @probVal, $probValue;
		}
	}

    $i=1;
    while(-e "${scoringDirectory}s${setNumber}bak$i.${DAT}") {$i++;}      ##don't overwrite existing backups
	$bakFileName ="${scoringDirectory}s${setNumber}bak$i.${DAT}";

	&createFile($bakFileName, 0660, $Global::numericalGroupID);
    open(RAWSCORES,">$bakFileName")  or &wwerror("$0","Can't open $bakFileName");
    &backup("s${setNumber}ful");						## make upto three backups

    &createFile("${scoringDirectory}s${setNumber}ful.${DAT}", $Global::scoring_files_permission, $Global::numericalGroupID);
	open(FULLSCORES,">${scoringDirectory}s${setNumber}ful.${DAT}")  or
	  &wwerror("$0","Can't open ${scoringDirectory}s${setNumber}ful.${DAT}");
    &backup("s${setNumber}scr");						## make upto three backups

  	&createFile("${scoringDirectory}s${setNumber}scr.${DAT}", $Global::scoring_files_permission, $Global::numericalGroupID);
    open(SCORES,">${scoringDirectory}s${setNumber}scr.${DAT}")   or
	  &wwerror("$0","Can't open ${scoringDirectory}s${setNumber}scr.${DAT}");
    $noOfProbs=@probList;
    print "noOfProbs is $noOfProbs\n" if $debugOn;
    print "problems values are @probVal\n"  if $debugOn;

    &bak_dat_header($setNumber, $noOfProbs , $dueDate, $dueTime, \@probVal);
    &ful_dat_header($setNumber, $noOfProbs , $dueDate, $dueTime, \@probVal);
    &scr_dat_header($setNumber, $noOfProbs , $dueDate, $dueTime, \@probVal);


    foreach $pin (@pinNumbersArray) {
        attachProbSetRecord($pin);
        my $studentLogin = getStudentLogin($pin);
        attachCLRecord($studentLogin);

        #### initialize arrays
        @bakArray=($pin,&CL_getStudentID($studentLogin), &CL_getStudentLastName($studentLogin), &CL_getStudentFirstName($studentLogin),&CL_getClassSection($studentLogin),&CL_getClassRecitation($studentLogin));
        @fulArray=@bakArray;
	    shift @fulArray;
        @scrArray=@fulArray;

        @Yarray=(0) x ($noOfProbs+1);           ##holds number of correct responses
        @Narray=(0) x ($noOfProbs+1);           ##holds number of incorrect responses
        @Sarray=(0) x ($noOfProbs+1);           ##holds scores

	    if ($method eq 'scofiles') {
		    ########  GET NUMBER OF CORRECT AND INCORRECT ANSWERS FROM SCO FILES #####
		    my $Yarrayref = \@Yarray;
		    my $Narrayref = \@Narray;
		    my $Sarrayref = \@Sarray;
		    &getRecordedScores($Sarrayref,$Yarrayref,$Narrayref,$pin);
		}
	    else {
		    ########  GET NUMBER OF CORRECT AND INCORRECT ANSWERS FROM DATABASE #####
		    for ($i=1;$i<=$noOfProbs;$i++) {
			    $Yarray[$i] = &getProblemNumOfCorrectAns($i,$pin);
			    $Narray[$i] = &getProblemNumOfIncorrectAns($i,$pin);
    			$Sarray[$i] = &getProblemStatus($i,$pin);
    		}
		}

        $warnTag=0;
        for ($i=1;$i<=$noOfProbs;$i++) {
            if ($Yarray[$i] > 99) {
                    $warnTag=1;
                    $Yarray[$i] = 99
            }
            if ($Narray[$i] > 99) {
                    $warnTag=1;
                    $Narray[$i] = 99
            }
        }
        if ($warnTag == 1) {
            &scoreMessage("Warning: more than 99 attempts for a single problem in ${scoringDirectory}s${setNumber}ful.${DAT}");
        }

        for ($i=1;$i<=$noOfProbs;$i++)  {
            push(@bakArray,$Sarray[$i]);
            push(@bakArray,$Yarray[$i]);
            push(@bakArray,$Narray[$i]);
            push(@fulArray,$Sarray[$i]);
            push(@fulArray,$Yarray[$i]);
            push(@fulArray,$Narray[$i]);
            push(@scrArray,$Sarray[$i]);
        }
        $dataLine=join("${DELIM}",@bakArray);
        print "$dataLine\n" if $debugOn;
        print RAWSCORES "$dataLine\n";
        $dataLine=join("${DELIM}",@fulArray);
        print "$dataLine\n" if $debugOn;
        print FULLSCORES "$dataLine\n";
        $dataLine=join("${DELIM}",@scrArray);
        print "$dataLine\n" if $debugOn;
        print SCORES "$dataLine\n";

    }                                               ##end of $pin for each loop
    close(RAWSCORES);
    close(FULLSCORES);
    close(SCORES);

    ##permissions set here

    chmod($Global::scoring_bak_files_permission,$bakFileName)  or
	  &wwerror("$0","Can't do chmod($Global::scoring_bak_files_permission,$bakFileName)");
    %assocArray = &dat2aa("s${setNumber}ful");
    &aa2dat(\%assocArray,"s${setNumber}ful","AbySec");                           ##alphbatize

    %assocArray = &dat2aa("s${setNumber}scr");
    &aa2dat(\%assocArray,"s${setNumber}scr","AbySec");                           ##alphbatize

}




sub ful_dat_header {
    ## add header lines to the set ful.${DAT} files
    my ($setNumber, $noOfProbs , $dueDate, $dueTime, $probValref) = @_;
    my $num =$noOfProbs;
    my @valArray = @$probValref;

    my ($i);

    $num=3*$num+5;          ## number of fields is 5 more than twice number of problems
    my @array = ();
    my $dataLine ='';

	## first header line
    @array=(" ") x $num;
    $array[0]=$HL[1];
    $dataLine=join("${DELIM}",@array);
    print FULLSCORES "$dataLine\n";

	## second header line   set number
    @array=(" ") x $num;
    $array[0]=$HL[2];
    for ($i=5; $i < $num; $i=$i+3) {$array[$i]=$setNumber;}
    $dataLine=join("${DELIM}",@array);
    print FULLSCORES "$dataLine\n";

	## third header line    problem numbers
    @array=(" ") x $num;
    $array[0]=$HL[3];
    for ($i=5; $i < $num; $i=$i+3) {$array[$i]=int(($i-2)/3);$array[$i+1]=int(($i-2)/3);$array[$i+2]=int(($i-2)/3);}
    $dataLine=join("${DELIM}",@array);
    print FULLSCORES "$dataLine\n";

	## fourth header line
    @array=(" ") x $num;
    $array[0]=$HL[4];
    for ($i=5; $i < $num; $i=$i+3) {$array[$i]=$dueDate;}
    $dataLine=join("${DELIM}",@array);
    print FULLSCORES "$dataLine\n";

	## fifth header line
    @array=(" ") x $num;
    $array[0]=$HL[5];
    for ($i=5; $i < $num; $i=$i+3) {$array[$i]=$dueTime;}
    $dataLine=join("${DELIM}",@array);
    print FULLSCORES "$dataLine\n";

	## sixth header line
    @array=(" ") x $num;
    $array[0]=$HL[6];
    for ($i=5; $i < $num; $i=$i+3) {$array[$i]=shift(@valArray);}
    $dataLine=join("${DELIM}",@array);
    print FULLSCORES "$dataLine\n";

	## seventh header line
    @array=(" ") x $num;
    $array[0]=$HL[7];
    $array[1]='LAST NAME';
    $array[2]='FIRST NAMES';
    $array[3]='SECTION';
    $array[3]='RECITATION';
	for ($i=5; $i < $num; $i=$i+3) {
	    $array[$i]='STATUS';
    	$array[$i+1]='#corr';
    	$array[$i+2]='#incorr';
    }
    $dataLine=join("${DELIM}",@array);
    print FULLSCORES "$dataLine\n";

}




sub bak_dat_header  {
    ## add header lines to the set bak.${DAT} files
    my ($setNumber, $noOfProbs , $dueDate, $dueTime, $probValref) = @_;
    my $num =$noOfProbs;
    my @valArray = @$probValref;

    my ($i);

    $num=3*$num+6;          ## number of fields is 6 more than twice number of problems
    my@array = ();
    my$dataLine = '';

	## first header line
    @array=(" ") x $num;
    $array[0]="line1";
    $array[1]=$HL[1];
    $dataLine=join("${DELIM}",@array);
    print RAWSCORES "$dataLine\n";

	## second header line
    @array=(" ") x $num;
    $array[0]="line2";
    $array[1]=$HL[2];
    for ($i=6; $i < $num; $i=$i+3) {$array[$i]=$setNumber;}
    $dataLine=join("${DELIM}",@array);
    print RAWSCORES "$dataLine\n";

	## third header line
    @array=(" ") x $num;
    $array[0]="line3";
    $array[1]=$HL[3];
    for ($i=6; $i < $num; $i=$i+3) {$array[$i]=int(($i-3)/3);$array[$i+1]=int(($i-3)/3);$array[$i+2]=int(($i-3)/3);}
    $dataLine=join("${DELIM}",@array);
    print RAWSCORES "$dataLine\n";

	## fourth header line
    @array=(" ") x $num;
    $array[0]="line4";
    $array[1]=$HL[4];
    for ($i=6; $i < $num; $i=$i+3) {$array[$i]=$dueDate;}
    $dataLine=join("${DELIM}",@array);
    print RAWSCORES "$dataLine\n";

	## fifth header line
    @array=(" ") x $num;
    $array[0]="line5";
    $array[1]=$HL[5];
    for ($i=6; $i < $num; $i=$i+3) {$array[$i]=$dueTime;}
    $dataLine=join("${DELIM}",@array);
    print RAWSCORES "$dataLine\n";

	## sixth header line
    @array=(" ") x $num;
    $array[0]="line6";
    $array[1]=$HL[6];
    for ($i=6; $i < $num; $i=$i+3) {$array[$i]=shift(@valArray);}
    $dataLine=join("${DELIM}",@array);
    print RAWSCORES "$dataLine\n";

	## seventh header line
    @array=(" ") x $num;
    $array[0]="line7";
    $array[1]=$HL[7];
    $array[2]="LAST NAME";
    $array[3]="FIRST NAMES";
	$array[4]="SECTION";
	$array[5]="RECITATION";
    for ($i=6; $i < $num; $i=$i+3) {
        $array[$i]='STATUS';
    	$array[$i+1]="#corr";
    	$array[$i+2]="#incorr";
    }
    $dataLine=join("${DELIM}",@array);
    print RAWSCORES "$dataLine\n";
}




sub scr_dat_header	{
    my ($setNumber, $noOfProbs , $dueDate, $dueTime, $probValref) = @_;
    my $num =$noOfProbs;
    my @valArray = @$probValref;

    my ($i);

    $num=$num+5;            ## number of fields is 5 more than number of problems
    my @array =();
    my $dataLine ='';

	## first header line
    @array=(" ") x $num;
    $array[0]=$HL[1];
    $dataLine=join("${DELIM}",@array);
    print SCORES "$dataLine\n";

	## second header line
    @array=(" ") x $num;
    $array[0]=$HL[2];
    for ($i=5; $i < $num; $i++) {$array[$i]=$setNumber;}
    $dataLine=join("${DELIM}",@array);
    print SCORES "$dataLine\n";

	## third header line
    @array=(" ") x $num;
    $array[0]=$HL[3];
    for ($i=5; $i < $num; $i++) {$array[$i]=$i-3;}
    $dataLine=join("${DELIM}",@array);
    print SCORES "$dataLine\n";

	## fourth header line
    @array=(" ") x $num;
    $array[0]=$HL[4];
    for ($i=5; $i < $num; $i++) {$array[$i]=$dueDate;}
    $dataLine=join("${DELIM}",@array);
    print SCORES "$dataLine\n";

	## fifth header line
    @array=(" ") x $num;
    $array[0]=$HL[5];
    for ($i=5; $i < $num; $i++) {$array[$i]=$dueTime;}
    $dataLine=join("${DELIM}",@array);
    print SCORES "$dataLine\n";

	## sixth header line
    @array=(" ") x $num;
    $array[0]=$HL[6];
    for ($i=5; $i < $num; $i++) {$array[$i]=shift(@valArray);}
    $dataLine=join("${DELIM}",@array);
    print SCORES "$dataLine\n";

	## seventh header line
    @array=(" ") x $num;
    $array[0]=$HL[7];
    $array[1]="LAST NAME";
    $array[2]="FIRST NAMES";
    $array[3]="SECTION";
    $array[4]="RECITATION";
	for ($i=5; $i < $num; $i++) {$array[$i]="STATUS";}
    $dataLine=join("${DELIM}",@array);
    print SCORES "$dataLine\n";

}



sub backup  {
	## takes as a parameter the filename without the extension, e.g. ("s5ful") or ("s5scr")
	## makes upto three backups of file with x, y, or z appended to filename where x
	## the most recent backup

    my $fileName =$_[0];
    if (-e "${scoringDirectory}${fileName}y.${DAT}") {
        rename("${scoringDirectory}${fileName}y.${DAT}","${scoringDirectory}${fileName}z.${DAT}") or
	      &wwerror("$0","can't rename ${scoringDirectory}${fileName}y.${DAT}");
    }

    if (-e "${scoringDirectory}${fileName}x.${DAT}") {
        rename("${scoringDirectory}${fileName}x.${DAT}","${scoringDirectory}${fileName}y.${DAT}") or
	      &wwerror("$0","can't rename ${scoringDirectory}${fileName}x.${DAT}");
    }

	if (-e "${scoringDirectory}${fileName}.${DAT}") {
       	rename("${scoringDirectory}${fileName}.${DAT}","${scoringDirectory}${fileName}x.${DAT}") or
	      &wwerror("$0","can't rename ${scoringDirectory}${fileName}.${DAT}");
    }
}

sub dat2aa  {
    ## takes as a parameter the filename without the extension, e.g. ("s5ful") or ("s5scr")
    my $fileName =$_[0];
    my(%aaArray);

    &checkdat("$fileName");
    print "dat2aa working on ${scoringDirectory}${fileName}.${DAT}\n" if $debugOn;
    %aaArray = &delim2aa("${scoringDirectory}${fileName}.${DAT}");
}



sub aa2dat {

	## Saves an associative array version of the scoring files as a delimited file.
    ## Takes three parameters.  The first is the name of an associative array
	## passed by reference. The second is a filename without the extension,
    ## e.g. s5ful or s5scr.  The third optional parameter determined the
    ## sorting order.  The default will be to sort alpabetically first by section,
	## then by student name. If the third parameter is 'A', the
    ## sort order will be alpabetically by student name.  If the third parameter is 'SbySec',
	## the sort order will be first by section, then by studentID. If the third parameter
	## is 'S', the sort order will be by studentID.
	## Creates a $filename.$DAT file which is an text version of the associative array
	## E.g., &aa2delim(\%aaArray, s1scr, A)

    my($aaArrayref,$fileName,$sortorder)=@_;
    %sortaaArray = %$aaArrayref;
    print "aa2delim working on ${scoringDirectory}${fileName}.${DAT}\n" if $debugOn;

    my(@sortedkeys,@keys,@hdrarray,%mark,$dbString,$studID);
    unless (defined $sortorder) {$sortorder = 'AbySec';}

	unless(-e "${scoringDirectory}${fileName}.${DAT}") {
	    &createFile("${scoringDirectory}${fileName}.${DAT}", $Global::scoring_files_permission, $Global::numericalGroupID);
	}
    open(DATFILE,">${scoringDirectory}${fileName}.${DAT}") or
	  &wwerror("$0","can't open ${scoringDirectory}${fileName}.${DAT}");
    @hdrarray=@HL;                          ##get header line labels
    shift(@hdrarray);                       ##put first header line label in 0th position
    foreach $studID (@hdrarray)   {            ##put header lines in ${DAT} file
        $dbString=$sortaaArray{$studID};
        $dbString="$studID${DELIM}$dbString";
        print DATFILE "$dbString\n";
    }
    if ($sortorder eq "S") {@sortedkeys = sort by_studentID (keys(%sortaaArray));}
	elsif ($sortorder eq "SbySec") {@sortedkeys = sort by_section_then_by_studentID (keys(%sortaaArray));}
	elsif ($sortorder eq "A") {@sortedkeys = sort by_studentName (keys(%sortaaArray));}
	elsif ($sortorder eq "SbyRec") {@sortedkeys = sort by_recitation_then_by_studentID (keys(%sortaaArray));}
	elsif ($sortorder eq "AbyRec") {@sortedkeys = sort by_recitation_then_by_studentName (keys(%sortaaArray));}
	else {@sortedkeys = sort by_section_then_by_studentName (keys(%sortaaArray));}

    grep($mark{$_}++,@hdrarray);            ##remove header line keys from sortedkeys
    @keys=grep(!$mark{$_},@sortedkeys);
    foreach $studID (@keys) {
        $dbString=$sortaaArray{$studID};
        $dbString="$studID${DELIM}$dbString";
    print DATFILE "$dbString\n";
    }
    close(DATFILE);

    &columnPrint ("${scoringDirectory}${fileName}.${DAT}","${scoringDirectory}${fileName}.${DAT}")
}

sub by_studentID {$a cmp $b;}                      ## sort by student ID

sub by_section_then_by_studentID {                        ## sort by student ID

	(&get_section($a) cmp &get_section($b))
	or
	($a cmp $b);
}

sub by_recitation_then_by_studentID {                        ## sort by student ID

	(&get_recitation($a) cmp &get_recitation($b))
	or
	($a cmp $b);
}


sub by_studentName {
    ## sort by student name
    (lc(&get_last_name($a)) cmp lc(&get_last_name($b)))
	or
	(lc(&get_first_names($a)) cmp lc(&get_first_names($b)))
	or
	($a cmp $b);
}

sub by_section_then_by_studentName {
    ## sort by student name
	(&get_section($a) cmp &get_section($b))
	or
    (lc(&get_last_name($a)) cmp lc(&get_last_name($b)))
	or
	(lc(&get_first_names($a)) cmp lc(&get_first_names($b)))
	or
	($a cmp $b);
}

sub by_recitation_then_by_studentName {
    ## sort by student name
	(&get_recitation($a) cmp &get_recitation($b))
	or
    (lc(&get_last_name($a)) cmp lc(&get_last_name($b)))
	or
	(lc(&get_first_names($a)) cmp lc(&get_first_names($b)))
	or
	($a cmp $b);
}

sub get_last_name {
	my ($studID) =@_;
	my @temp=split(/${DELIM}/,$sortaaArray{$studID});
	return $temp[0] if defined $temp[0];
	'';
}
sub get_first_names {
    my ($studID) =@_;
    my @temp=split(/${DELIM}/,$sortaaArray{$studID});
	return $temp[1] if defined $temp[1];
	'';
}
sub get_section {
    my ($studID) =@_;
    my @temp=split(/${DELIM}/,$sortaaArray{$studID});
	return $temp[2] if defined $temp[2];
	'';
}

sub get_recitation {
    my ($studID) =@_;
    my @temp=split(/${DELIM}/,$sortaaArray{$studID});
	return $temp[3] if defined $temp[3];
	'';
}
sub append_score_db {
	## Takes two parameters. The first is the filename (without the extension)
    ## of the cummulative database. The second  is the filename (without the
    ## extension) of the database to be appended. E.g.  append_score_db ("dbfile", "appenddbfile")

    my ($dbFile,$appenddbFile)=@_;
    my ($studID,$temp,$i,$dbNoOfFields,$appenddbNoOfFields,$dbString,$string);
    my (@dbArray,@temp,@temp1,%appenddbaArray);
    my (%dbaArray);
	%dbaArray = &dat2aa("$dbFile");
    my $cumm_db_non_empty =	scalar(%dbaArray); ## this will be 0 if empty
    %appenddbaArray = &dat2aa("$appenddbFile");

    if ($cumm_db_non_empty) {
        $_=$dbaArray{"NO OF FIELDS"};
        $temp=s/${DELIM}/${DELIM}/g;            ##find number of delims in  string
    }
    else {$temp =0};
    $dbNoOfFields=$temp+2;                  ##num of fields in dat file
    $_=$appenddbaArray{"NO OF FIELDS"};
    $temp=s/${DELIM}/${DELIM}/g;            ##find number of delims in  string
    $appenddbNoOfFields=$temp+2;
    foreach $studID (keys %dbaArray) {
    	if ( not defined ($appenddbaArray{$studID})) {  ##handle case where record is not in
        	##appended database
            $dbString=$dbaArray{$studID};
            @dbArray=&getRecord($dbString);
            &scoreMessage("No record for $dbArray[1] $dbArray[0] , ID = $studID,\n in the $appenddbFile database.  The record will be padded with blanks.");
            $temp=" ${DELIM}" x ($appenddbNoOfFields-5);     ##blanks for fields
            $dbString.=$temp;
            $dbaArray{$studID}=$dbString;

        }
        else {
            $dbString=$appenddbaArray{$studID};
            $dbString=~s/^[^${DELIM}]*${DELIM}[^${DELIM}]*${DELIM}[^${DELIM}]*${DELIM}[^${DELIM}]*//;       ##remove last and first names
																##and section ##and (8/16/000 recitation
            $dbaArray{$studID}.=$dbString;
        }
    }
                                            ##end of foreach

    foreach $studID (keys (%appenddbaArray)) {          ##handle case where record is not in
                                          ##cummulative database
    	unless (defined ($dbaArray{$studID})) {
        	$dbString=$appenddbaArray{$studID};
		 	@dbArray=&getRecord($dbString);
            &scoreMessage("No record for $dbArray[1] $dbArray[0] , ID = $studID,\n in the $dbFile database.  The record will be padded with blanks.") if $cumm_db_non_empty ;
            $temp=" ${DELIM}" x ($dbNoOfFields-5);
            $dbString=~s/(^[^${DELIM}]*${DELIM}[^${DELIM}]*${DELIM}[^${DELIM}]*${DELIM}[^${DELIM}]*${DELIM})/${1}$temp/;
            $dbaArray{$studID}=$dbString;
        }                               ##end of unless

    }                                       ##end of foreach

	&backup("$dbFile");                             ##backup dat files
    &aa2dat(\%dbaArray,"$dbFile","AbySec");               ##update dat file
}                                               ##end of sub append_score_db




sub checkdat {
	## takes as a parameter the filename without the extension, e.g. ("S5ful") or ("S5scr")

    my $fileName =$_[0];
    my $msg = htmlcheckdat($fileName);
    unless ($msg eq 'OK') {
        &wwerror("$0","$msg");
    }
}

sub htmlcheckdat {
	## takes as a parameter the filename without the extension, e.g. ("S5ful") or ("S5scr")
    ## returns 'OK' or error message depending on whether dat file is valid or not

    my $fileName =$_[0];
    my ($noOfDelim,$firstLine,$dbString,$num,$temp,$i);
    my (@keyList, $msg);

    open(DATFILE,"${scoringDirectory}${fileName}.${DAT}") or
	  &wwerror("$0","can't open ${scoringDirectory}${fileName}.${DAT}");
    $_= <DATFILE>;
    $firstLine=$_;
    if (defined $firstLine) {
        $noOfDelim=s/$DELIM/$DELIM/g;
        while (<DATFILE>) {
    	    $dbString=$_;
            $num=s/$DELIM/$DELIM/g;
            if ($num != $noOfDelim) {
                $msg = "${scoringDirectory}${fileName}.${DAT} is corrupted.\n The record\n  $firstLine\n
 			     contains $noOfDelim  delimiters ($DELIM) whereas the record\n
 			     $dbString  contains $num  delimiters.\n";
 			    return $msg;
 		    }
        }
    }
    close DATFILE;
    open(DATFILE,"${scoringDirectory}${fileName}.${DAT}") or
	  &wwerror("$0","can't open ${scoringDirectory}${fileName}.${DAT}");
    @keyList=();
    while (<DATFILE>) {
    	chomp;
    	s/${DELIM}.*//;                         ## get key
    	push(@keyList,$_);
    }
    close DATFILE;
    @keyList = sort(@keyList);
    for ($i=0; $i < @keyList-1; $i++) {
    	if ($keyList[$i] eq $keyList[$i+1]) {
	        $msg = "duplicate keys equal to $keyList[$i] in ${scoringDirectory}${fileName}.${DAT}";
	        return $msg;
        }
    }
    $msg ='OK';
    return $msg;
}



sub recover  {
    ## takes as a parameter the filename without the extension of the bak data file,
    ## e.g. ("s5bak3").   Produces new versions of the corresponding ful and scr files,
    ## e.g. s5ful and s5scr.
    my $fileName =$_[0];
    my(@dbArray,$dbString,$fulFileName,$scrFileName,$temp,$noOfProbs,$noOfDelim,$i,$j,$ext);
	my (%assocArray);
    $temp=$fileName;
    $temp=~s\bak.*\\;
    $fulFileName="$temp"."ful";
    $scrFileName="$temp"."scr";

    &checkdat("$fileName");

	&backup("$fulFileName");
	&backup("$scrFileName");


    open(BAKFILE,"${scoringDirectory}${fileName}.${DAT}") or
      &wwerror("$0","can't open ${scoringDirectory}${fileName}.${DAT}");
    &createFile("${scoringDirectory}${fulFileName}.${DAT}", $Global::scoring_files_permission, $Global::numericalGroupID);
    open(FULLSCORES,">${scoringDirectory}${fulFileName}.${DAT}") or
      &wwerror("$0","can't open ${scoringDirectory}${fulFileName}.${DAT}");
    &createFile("${scoringDirectory}${scrFileName}.${DAT}", $Global::scoring_files_permission, $Global::numericalGroupID);
    open(SCORES,">${scoringDirectory}${scrFileName}.${DAT}") or
      &wwerror("$0","can't open ${scoringDirectory}${scrFileName}.${DAT}");

                                   ##recover ful score file
    while (<BAKFILE>) {
    	chomp;
        @dbArray=&getRecord($_);
        shift(@dbArray);
        $dbString=join("${DELIM}",@dbArray);
        print FULLSCORES "$dbString\n";
    }                       ## ful score file recovered

                                        ## recover scr score file

   open(BAKFILE,"${scoringDirectory}${fileName}.${DAT}") or
      &wwerror("$0","can't open ${scoringDirectory}${fileName}.${DAT}");
    while (<BAKFILE>) {
    	chomp;
        @dbArray=&getRecord($_);
        shift(@dbArray);            ## throw away first element
        my @outArray =();
        for ($j=1; $j <= 5; $j++) { ## put back next 5 elements
            push @outArray,shift(@dbArray);
        }
        while (@dbArray) {
            push @outArray,shift(@dbArray);     #save
            shift(@dbArray);        ## throw away
            shift(@dbArray);        ## throw away
        }
        $dbString=join($DELIM,@outArray);
        print SCORES "$dbString\n";
    }                       ## scr score file recovered

    close(BAKFILE);
    close(FULLSCORES);
    close(SCORES);

    %assocArray = &dat2aa("${fulFileName}");
    &aa2dat(\%assocArray,"${fulFileName}","AbySec");                           ##alphbatize

    %assocArray = &dat2aa("${scrFileName}");
    &aa2dat(\%assocArray,"${scrFileName}","AbySec");                           ##alphbatize
}


sub delete_columns {
    ## Takes four parameters. The first is the filename without the extension,
    ## the second is the first column of a range of columns to be deleted.
	## the third is the last column of a range of columns to be deleted.
    ## the fourth is 1 for a prompt and 0 for none
	## e.g. (s5ful,20,30,1) or (s5scr,10 15,0).
    ## Note that "ful" files contain 2 columns for each problem (a `#corr' and an `#incorr'
    ## column) where as "scr" files contain 1 column for each problem (a `score' column).

	## the first column is 1 (not 0).  -2 refers to the second to last column, etc

    my ($fileName,$beginCol,$endCol,$prompt)=@_;
    my (@dbArray,@fileArray,$dbString,$studID,$temp,$temp1,$i,$tempFileName);
	my ($lastCol,$negBeginCol,$negEndCol);

   	&checkdat("$fileName");
    open(DATFILE,"${scoringDirectory}${fileName}.${DAT}") or
      &wwerror("$0","can't open ${scoringDirectory}${fileName}.${DAT}");
    $_ = <DATFILE>;
    chomp;
    @dbArray=&getRecord($_);
	$lastCol = @dbArray;
	close(DATFILE);
	if ($beginCol =~ /^ *-/) {
		$negBeginCol = $beginCol;
		$beginCol = $lastCol + $beginCol +1;
	}
	else {$negBeginCol = $beginCol-$lastCol-1;}

	if ($endCol =~ /^ *-/) {
		$negEndCol = $endCol;
		$endCol = $lastCol + $endCol +1;
	}
	else {$negEndCol = $endCol-$lastCol-1;}

	if ($prompt) {
        print "\n\nThis procedure will remove a range of columns from the ${scoringDirectory}${fileName}.${DAT}\n";
        print "data set. That is a range of fields in each record in the ${fileName}.${DAT} data set will be removed.\n\n";
		print "The first column to be removed will be column $beginCol (i.e. column $negBeginCol).\n";
		print "The last column to be removed will be column $endCol (i.e. column $negEndCol).\n\n";
		print "Note that the first column is column 1 (i.e. column -$lastCol) and\n";
		print "the last column is column $lastCol (i.e. column -1).\n\n";
        print "Note also that \"ful\" data sets (e.g. \"s5ful\") contain 2 columns for each problem\n";
        print "(a `#corr' and an `#incorr' column) where as \"scr\" data sets (e.g. \"s5scr\") contain 1 column\n";
        print "for each problem (a `score' column).\n";
        print "DO YOU WANT TO CONTINUE WITH THIS OPERATION? (Y or N)\n";
        $temp=<STDIN>;
        chomp($temp);
        unless (($temp eq "Y") || ($temp eq "y")) {
            print "Operation aborted\n";
            exit;
        }
    }
	unless ((1 <= $beginCol) && ($beginCol <= $endCol) && ($endCol <= $lastCol)) {
	    &wwerror("$0","Invalid Column Range.  Operation aborted\n
	      beginCol =$beginCol   endCol = $endCol   lastCol = $lastCol\n");
        exit;
    }
	&backup("$fileName");

	##copy backup file back to original file

	open(BACKFILE,"${scoringDirectory}${fileName}x.${DAT}")  or
      &wwerror("$0","can't open ${scoringDirectory}${fileName}x.${DAT}");
	@fileArray = <BACKFILE>;
	open(ORGFILE,">${scoringDirectory}${fileName}.${DAT}")  or
      &wwerror("$0","can't open ${scoringDirectory}${fileName}.${DAT}");
	print ORGFILE @fileArray;
	close(ORGFILE);

	##find unique temp file name
 	$i=1;
    $tempFileName = "${scoringDirectory}$fileName$i";
    while(-e "$tempFileName") {
        $i++;
        $tempFileName = "${scoringDirectory}$fileName$i";
    }

	open(TEMPFILE,">$tempFileName")  or
      &wwerror("$0","can't open $tempFileName");
    open(DATFILE,"${scoringDirectory}${fileName}.${DAT}")  or
      &wwerror("$0","can't open ${scoringDirectory}${fileName}.${DAT}");
    while (<DATFILE>) {
        chomp;
        @dbArray=&getRecord($_);
        splice(@dbArray,$beginCol-1,$endCol-$beginCol+1);
		$dbString=join("${DELIM}",@dbArray);
		print TEMPFILE "$dbString\n";
    }

    close(DATFILE);
	close(TEMPFILE);
	&columnPrint("$tempFileName","$tempFileName");
    rename("$tempFileName","${scoringDirectory}${fileName}.${DAT}") or
      &wwerror("$0","can't rename ${scoringDirectory}${fileName}.${DAT}");
	chmod($Global::scoring_files_permission, "${scoringDirectory}${fileName}.${DAT}") or
      &wwerror("$0","Can't do chmod($Global::scoring_files_permission, ${scoringDirectory}${fileName}.${DAT})");
    chown(-1,$Global::numericalGroupID,"${scoringDirectory}${fileName}.${DAT}") or
      &wwerror("$0","Can't do chown(-1,$Global::numericalGroupID,${scoringDirectory}${fileName}.${DAT})");
}		############  end of delete_columns;


sub total_score  {
    ## Takes two parameters, the filename (without the extension) of the score
    ## database and a caption for the total column. E.g. ("s5scr", "set 5").
	## Computes the total score by calculating the sum over
    ## columns 5,6, ..., last column of round_score((prob value)*(prob scr)).
    ## That is value of each problem is rounded then added.  Note that col. 1
    ## is the student id, col.'s 2 and 3 hold the student last and first names,
	## and column 4 holds the section number.
    ## The total is appended as a new col in the score database with a prob value of 0
    ## so that it will not contribute to future calculations of the total score.

    my($dbFile,$caption) = @_;

	my (%assocArray);
    my($studID,$i,$dbNoOfDelim,$dbString,$total);
    my(@dbArray,@valArray);


	%assocArray =&dat2aa("$dbFile");                ## create assoc array
    &backup("$dbFile");                             ## backup dat file
    $_=$assocArray{"NO OF FIELDS"};
    $dbNoOfDelim=s/${DELIM}/${DELIM}/g;             ## find number of delims in  string
    $dbString=$assocArray{"PROB VALUE"};
	@valArray=&getRecord($dbString);          ## save prob values

    foreach $studID (keys %assocArray) {
    	$dbString=$assocArray{$studID};
		@dbArray = &getRecord($dbString);

        if (($studID eq "NO OF FIELDS")  || ($studID eq "PROB NAME") || ($studID eq "DUE DATE")|| ($studID eq "DUE TIME")) {
        	push (@dbArray, " ");
        }
        elsif  ($studID eq "SET NUMBER") {push (@dbArray,$caption);}   ## set set number to caption
        elsif ($studID eq "PROB VALUE") {push (@dbArray, 0);}  ## set prob val to zero for total column
        elsif ($studID eq "STUDENT ID") {push (@dbArray, "total");}
        else {       ## handle actual data records and calculate total
        	$total=0;
            for ($i=4; $i <= $dbNoOfDelim; $i++)  {$total += round_score($dbArray[$i]*$valArray[$i]);}
            push (@dbArray, $total);
        }                               ## end of if statement
        $dbString=join("${DELIM}",@dbArray);
        $assocArray{$studID}=$dbString;
    }                                       ## end of foreach $studID
 	&aa2dat(\%assocArray,"${dbFile}","AbySec");           ##update dat file and alphbatize
}                                               ##end of sub total_score




sub test {
    ## Takes one parameter, the filename (without the extension) of the score
    ## database. E.g. ("s5scr").
    my($dbFile)=@_;
    my (%assocArray);
    %assocArray =&dat2aa("$dbFile");                ## create assoc array
    &backup("$dbFile");                             ## backup dat file
    &aa2dat(\%assocArray,"${dbFile}","AbySec");           ##update dat file and alphbatize
}                                               ##end of test


sub readProblemsAndValuesFromDB {
	my ($setNum,$studID,$cgiMode,$batchMode) = @_;

#print "cgiMode is $cgiMode\n";
#print "setNum is $setNum\n";
#print "studID is $studID\n";
#print "batchMode is $batchMode\n";


	my @keyList = &getAllProbSetKeysForSet($setNum);
	my (@problemList,@problemValues,@fixedProblemList,$fixedProblemValuesref,$num);
    unless ($studID) {$studID = $keyList[0];}  ##take a random studID if studID not defined
    &attachProbSetRecord($studID) || &wwerror("$0","no Record $studID");
    my $dueDate = &getDueDate($studID);
	my $fixedLogin_name = &getStudentLogin($studID);
	my ($problemListref,$problemValuesref) = getProblemsAndValues($studID);
	@fixedProblemList = @$problemListref;
	$fixedProblemValuesref = $problemValuesref;
	my $noOfProbs = @fixedProblemList;
	my $warningMsg = '';

	unless ($batchMode) {
    	foreach $studID (@keyList) {
	    	($problemListref,$problemValuesref) = getProblemsAndValues($studID);
		    @problemList = @$problemListref;
		    unless (($noOfProbs == @problemList) and (&arraysAreEqual($fixedProblemValuesref,$problemValuesref))) {
		        my $login_name = &getStudentLogin($studID);
		        my $warningMsg = "
The values for problems (or the number of problems) are not the same for all students
For example, the values for problems for loginID $fixedLogin_name are:
@$fixedProblemValuesref
whereas the values for problems for loginID $login_name are:
@$problemValuesref
You can Continue using the values for loginID $fixedLogin_name for all students or you can Quit";

			    if ($cgiMode) {return($warningMsg, $dueDate, \@fixedProblemList, $fixedProblemValuesref); }
			    else {
			        print $warningMsg;
		        	print "\nEnter C or Q:[C]";
			        my $ans;
                    $ans=<STDIN>;
                    chomp($ans);
                    if (($ans eq "Q") || ($ans eq "q")) {exit 0;}
                }
            }
        }
	}
	($warningMsg, $dueDate, \@fixedProblemList, $fixedProblemValuesref);
}



sub getProblemsAndValues {
	my ($studID) = @_;
	&attachProbSetRecord($studID) || &wwerror("$0","no Record $studID");
	my @problems = &getAllProblemsForProbSetRecord($studID);
	my @problemList = sort( { $a <=> $b }  @problems);
	my @problemValues = ();
	my ($num, $problemListref, $problemValuesref);
	foreach $num (@problemList) {push @problemValues, &getProblemValue($num,$studID)}
	$problemListref = \@problemList;
	$problemValuesref= \@problemValues;
	($problemListref, $problemValuesref);
}


sub arraysAreEqual {
    my ($arrayOneref, $arrayTworef) = @_;
	my $i;
	unless ( @$arrayOneref == @$arrayTworef) { return 0 ;}
	for ($i =0; $i < $#{@$arrayOneref}+1 ;$i++) {
		unless ( $$arrayOneref[$i] == $$arrayTworef[$i]) { return 0 ;}
    }
	1;
}

sub getRecordedScores {

    my ($Sarrayref,$Yarrayref,$Narrayref,$psvn) =  @_;
    &attachProbSetRecord($psvn);
    my $setNumber   =   &getSetNumber($psvn);
    &detachProbSetRecord($psvn);
        my ($scoreFileName)="${databaseDirectory}$scoreFilePrefix$setNumber$dash${psvn}.sco";
        #print  "Reading from file $scoreFileName\n" if $debugON;
        if ( open(SCORE_FILE,"<$scoreFileName")  )  {
            while (<SCORE_FILE>)    {
                my @temp=split(/$DELIM/,$_);
                $$Sarrayref[$temp[0]] = $temp[1];
                $$Yarrayref[$temp[0]] = $temp[2];
                $$Narrayref[$temp[0]] = $temp[3];

            };
            close(SCORE_FILE);
        }   else {
          warn "Warning: Couldn't open $scoreFileName.  Will continue.\n";
        }
     #  OPERATES ON THE ARRAYS Yarray and Narray.
}
